/************************************************
 *
 * @author Daniela da Cruz
 * @version 2.0, September 2006 
 *
 * This class is the second frame that appears
 * in Alma System, and it's here were we
 * made the steps managment over a program
 *
 ************************************************/

package Alma;

import Alma.animation.Actor;
import Alma.animation.DrawingPanel;
import  Lisa.Parser.CSyntaxTree;

import  java.awt.Color;
import  java.awt.Point;
import  java.awt.Dimension;
import  java.awt.BorderLayout;
import  java.awt.event.KeyEvent;
import  java.awt.event.ActionEvent;

import  java.util.ArrayList;

import  javax.swing.JFrame;
import  javax.swing.JSplitPane;
import  javax.swing.JScrollPane;
import  javax.swing.JTextPane;
import  javax.swing.JPanel;
import  javax.swing.JButton;
import  javax.swing.JMenu;
import  javax.swing.JMenuItem;
import  javax.swing.JMenuBar;
import  javax.swing.JLabel;
import  javax.swing.JOptionPane;
import  javax.swing.JFileChooser;
import  javax.imageio.ImageIO;
import  javax.swing.KeyStroke;

import  java.awt.image.BufferedImage;

import  java.io.File;

public class AnimateFrame extends JFrame
{
	private JFrame frame;
	private Graph graph;
	private Steps stepsT;
	private TableDesign table;
	private TAB tab;
	private ArrayList vertices;
	private double zoom;
	private JMenu menu;	
	private JMenuBar menuBar;		
	private JMenuItem menuItem;				
	private JButton back, forward, reset, group;
	private String fileName;
	private JFileChooser fileChooser;
	private Backend backend;
	private CSyntaxTree root;
	private Visualize vis;
	private JSplitPane vertical;
	private Motor motor;
	private JTextPane editorPane;
	private JScrollPane right;
    //This is the panel to draw the images of the prob Domain
    private DrawingPanel animationPanel;

	public AnimateFrame()
	{
		/** initializating varibles of class **/
		this.frame           = new JFrame();
		this.stepsT          = new Steps();
		this.graph           = new Graph();
		this.table           = new TableDesign();
		this.zoom            = 1.0;
		this.backend         = new Backend();
		this.vertices        = new ArrayList();
		this.vis             = new Visualize(this.graph);
		this.fileName	     = new String();
		this.motor           = new Motor();
		this.frame.setTitle("Alma - Program Visualization and Comprehension");
		this.frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.animationPanel      = new DrawingPanel();
	}

	/** makes window visible
	 * */
	public void setVisible()
	{
		this.frame.pack();
		this.frame.setExtendedState(this.frame.getExtendedState()|JFrame.MAXIMIZED_BOTH);
		//vertical.setDividerLocation((int)Toolkit.getDefaultToolkit().getScreenSize().getHeight());
		this.frame.setVisible(true);
        //At this point, since the animationPoint is a valid Component
        //in the Frame, it can be set right away in the backend to be passed
        //to the animation rules data base and so on
        this.backend.setAnimationPanel(this.animationPanel);
        this.backend.setFrame(this);
	}

    public void setActors(ArrayList<Actor> actors){
        this.animationPanel.setActors(actors);
        this.animationPanel.makeAnimation();
    }

	/**************************************
	 * Set file to show in left pane, south corner 
	 *
	 *
	**************************************/
	public void setFile(String name)
	{
		this.fileName = name;
	}

	/**************************************
	 *
	 * sets the syntax tree of our program
	 *
	 * ************************************/

	public void setRoot(CSyntaxTree root)
	{
		this.root = root;
	}

	/************************************************************
	 *
	 * animate our program
	 * 
	 * this method it is responsable by creat each step of rewrite 
	 * and visualization to, then, we control with actions
	 * with buttons
	 * 
	 * **********************************************************/
	 
	 public void createBackend()
	 {
		this.backend.firstStep(this.root,false,null,null,false,null,this.fileName,"Program");
		this.table.setTypes(((CRoot)this.root).getTableTypes());
	 }

	/**************************************
	 * 
	 * init all components of our frame
	 *
	 **************************************/	 
 	 public void initComponents()
	{
		/** local variable **/
		Table aux;

		ReadFile readFile      = new ReadFile();
		File testFile          = new File(this.fileName);
		JPanel panel           = new JPanel(new BorderLayout());
		JPanel left            = new JPanel(new BorderLayout());
		JPanel panelTable      = new JPanel(new BorderLayout());
		JPanel panelButton     = new JPanel();
		menu                   = new JMenu("File");
		menuBar                = new JMenuBar();
        this.animationPanel    = new DrawingPanel();

		JScrollPane p;
		JSplitPane horizontal;
		this.editorPane      = new JTextPane();
		this.editorPane.setEditorKit(new NumberedEditorKit());
		this.editorPane.setText(readFile.getContents(testFile));
		this.editorPane.setEditable(false);
		JScrollPane editor = new JScrollPane(this.editorPane);
		editor.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		editor.setPreferredSize(new Dimension(250, 445));
		editor.setMinimumSize(new Dimension(10, 10));

		/*************************************/

	  	// change the animationPanel steps obtained by backend.animate
		this.stepsT = backend.getAnimationSteps();
		
		// the table
		if (stepsT.getSteps()!= null)
		{
			if (stepsT.getSteps().size() > 0)
			{
				aux         = ((Table)stepsT.getSteps().get(0));
				table.createTable(aux.getTable());
				table.setCellColor(aux.getBefore(),aux.getNext());
			
				// the graph
				motor.makeGraph(aux.getAll(),vertices);
				graph.setGraph(vertices);
				if (aux.getRow() != -1) {
					//System.out.println("ROW196: " + aux.getRow());
					motor.highlightPanel(editorPane,aux.getRow());
				}
			}
			else {
				JOptionPane.showMessageDialog(frame, "No steps to animate!", "Alma information",
				      JOptionPane.INFORMATION_MESSAGE);
			}
		}

		
		menu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(menu);

		menuItem = new JMenuItem("Export graph (JPEG)");
		menuItem.setMnemonic(KeyEvent.VK_E);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E, ActionEvent.ALT_MASK));
		
		menu.add(menuItem);

		menuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) 
		{
			JFileChooser f = new JFileChooser();
			int returnVal = f.showSaveDialog(frame);
			File file;

			if (returnVal == JFileChooser.APPROVE_OPTION) 
			{
				file = f.getSelectedFile();
				//System.out.println(f.getCurrentDirectory());
				String fileName = file.getName();

				File out = new File(f.getCurrentDirectory()+"\\"+fileName+".jpg");
				Color bg = null; // Use this to make the background transparent
				bg = graph.getGraph().getBackground(); // Use this to use the graph background color
				BufferedImage img = graph.getGraph().getImage(bg, 3);

				try
				{
					ImageIO.write(img, "jpg", out);
					JOptionPane.showMessageDialog(frame,
				      "Graph exported to file " + fileName + ".jpg!",
				      "Alma information",
				      JOptionPane.INFORMATION_MESSAGE);
				}
				catch (Exception xe)
				{
				      JOptionPane.showMessageDialog(frame, "Empty graph! Graph not exported",
				      "Alma information",
				      JOptionPane.INFORMATION_MESSAGE);
				}
			} 
			else 
			{
				JOptionPane.showMessageDialog(frame, "Empty name of file! Graph not exported",
								      "Alma information",
								      JOptionPane.INFORMATION_MESSAGE);
			}
		}
		});
			
		menuItem = new JMenuItem("Close");
		menuItem.setMnemonic(KeyEvent.VK_X);		
		menu.add(menuItem);

		menuItem.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) 
		{
			//System.exit(0);
		}
        	});
				
		menu = new JMenu("Help");
		menu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(menu);

		menuItem = new JMenuItem("About");
		menuItem.setMnemonic(KeyEvent.VK_A);		
		menu.add(menuItem);
				
	
	
		/******************************************
		 * Reset button
		 * ***************************************/

		reset = new JButton("Reset");
		reset.setEnabled(false);

		reset.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
			back.setEnabled(false);
			forward.setEnabled(true);
			stepsT.setPosition(0);

			Table aux   = ((Table)stepsT.getSteps().get(0));
			motor.makeGraph(aux.getAll(),vertices);
			graph.setGraph(vertices);
			table.setCellColor(aux.getBefore(),aux.getNext());
          	}
        	});

	        /******************************************
		 * Backward button
		 * ***************************************/

		back = new JButton("Back");
		back.setEnabled(false);

		back.addActionListener(new java.awt.event.ActionListener() {
          	public void actionPerformed(java.awt.event.ActionEvent e) {		
          	}
        	});

		//group = new JButton("Group");
		//group.addActionListener(new java.awt.event.ActionListener() {
          	//public void actionPerformed(java.awt.event.ActionEvent e) {
		//}
		//});
		
	        /*****************************************
		 * Forward button
		 * ***************************************/

		forward = new JButton("Forward");
		forward.addActionListener(new java.awt.event.ActionListener() {
          	public void actionPerformed(java.awt.event.ActionEvent e) {
			ArrayList ids = new ArrayList();

			backend.forwardStep();
			Table t = backend.getAnimationSteps().getLast();

			// change graph
			vertices.clear();
			motor.makeGraph(t.getAll(),vertices);
			graph.setGraph(vertices);
			if (t.getRow() != -1) {
                //System.out.println("ROW334: " + t.getRow());
                motor.highlightPanel(editorPane, t.getRow());
            }

			Point p = new Point(t.getPosition().x-40,t.getPosition().y-40);
			right.getViewport().setViewPosition(p);
			// change identifiers table
			ids = t.getTable();
			if (ids.size() != 0) { 
				table.setTable(ids); 
				table.setCellColor(t.getBefore(),t.getNext());
			}	
		}
        	});

		/*****************************************
		 * Zoom + button
		 * ***************************************/

		JButton zoomM = new JButton("Zoom +");
		zoomM.addActionListener(new java.awt.event.ActionListener() {
          	public void actionPerformed(java.awt.event.ActionEvent e) {
			zoom += 0.1;
			graph.getGraph().setScale(zoom);
          	}
        	});

		/*****************************************
		 * Zoom - button
		 * ***************************************/

		JButton zoomL = new JButton("Zoom -");
		zoomL.addActionListener(new java.awt.event.ActionListener() {
          	public void actionPerformed(java.awt.event.ActionEvent e) {
			if (zoom != 0.1) {
				zoom -= 0.1;
				graph.getGraph().setScale(zoom);
			}
			else {
				JOptionPane.showMessageDialog(frame,"Zoom out of ranges!",
								    "Alma information",
								    JOptionPane.INFORMATION_MESSAGE);
			}
          	}
        	});


        //LAYOUT DEFINITION
        right   = new JScrollPane(graph.getGraph());

		p = new JScrollPane(table.getTable());
		p.setPreferredSize(new Dimension(150,180));
		panelTable.add(new JLabel("Identifiers",JLabel.CENTER),BorderLayout.NORTH);
		panelTable.add(p,BorderLayout.CENTER);

		panelTable.add(editor, BorderLayout.SOUTH);

        left.add(panelTable, BorderLayout.NORTH);

		//panelButton.add(reset);
		//panelButton.add(back);
        panelButton.add(zoomL);
		panelButton.add(forward);
		panelButton.add(zoomM);
		//panelButton.add(group);

        left.add(panelButton, BorderLayout.SOUTH);

        JPanel p1 = new JPanel(new BorderLayout());
        JSplitPane pright = new JSplitPane(JSplitPane.VERTICAL_SPLIT, right, animationPanel);
        pright.setDividerLocation(400);


		horizontal = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left, pright);
		horizontal.setDividerLocation(360);
        p1.add(horizontal);

		//graph.getGraph().setScale(0.4);
		right.getViewport().setViewPosition(new Point(500,10));
		this.frame.setJMenuBar(menuBar);
		this.frame.getContentPane().add(p1);
	}

}
