/**
 * @author Nuno Oliveira
 * @version 1.0
 * 
 * This class will represent an object that is actually
 * controlled by the program. The images and the state of the
 * object are here declared.
 * 
 */


package Alma.animation;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Actor {

	//private String figureName; //the name of the figure
    //private BufferedImage image; //the image of the actor
	private String label; //the label of the image
    private boolean isLabeled; //says whether there is label on the actor
    private HashMap<String, Object> state; //stores the state of the actor
    private ArrayList<BufferedImage> poses; // the poses (images) that an actor may assume
    private String[] figuresPath; // the name of the figures
    private int[] usedPoses; //poses that will be used to animate the object
                             //by default it uses the first pose.
    private boolean isAggregated = false;


    
    /**
     * @param figureName the name of the images, relative to Images directory
     * @param poses the 0-based list of poses to be used in the animation
     */
    public Actor(String[] figureName/*, int[] poses*/) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = false;
        this.label = "";
        this.state = new HashMap<String, Object>();
        this.state.put("x", 0);
        this.state.put("y", 0);
        this.state.put("angle", 0);
//        if(poses != null || poses.length > 0){
//            this.usedPoses = poses;
//        } else {
            this.usedPoses = new int[] {0};
        //}
    }


    /**
     *
     * @param figureName the anme of the images, relative to the Images directory
     * @param poses the 0-based list of poses to be used in the animation
     * @param state an hashmap of type (variable, value), representing <br/>
     * the inner state of the actor. variables <b>x</b>, <b>y</b> and <b>angle</b><br/>
     * are already defined, but can allways be redefined
     */
    public Actor(String[] figureName/*, int[] poses*/, HashMap<String, Object> state) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = false;
        this.label = "";
        this.state = new HashMap<String, Object>();
        this.state.put("x", 0);
        this.state.put("y", 0);
        this.state.put("angle", 0);
        this.state.putAll(state);
//        if(poses != null || poses.length > 0){
//            this.usedPoses = poses;
//        } else {
            this.usedPoses = new int[] {0};
        //}
    }

    /**
     *
     * @param figureName the anme of the images, relative to the Images directory
     * @param poses the 0-based list of poses to be ued in the animation
     * @param label the name of the variable which value is to be set as label
     * @param state an hashmap of type (variable, value), representing <br/>
     * the inner state of the actor. variables <b>x</b>, <b>y</b> and <b>angle</b><br/>
     * are already defined, but can allways be redefined
     */
    public Actor(String[] figureName/*, int[] poses*/, String label, HashMap<String, Object> state) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = true;
        this.label = label;
        this.state = new HashMap<String, Object>();
        this.state.put("x", 0);
        this.state.put("y", 0);
        this.state.put("angle", 0);
        this.state.putAll(state);
        /*if(poses != null || poses.length > 0){
            this.usedPoses = poses;
        } else {*/
            this.usedPoses = new int[] {0};
        //}
    }

    /**
     * @param figureName the anme of the images, relative to the Images directory
     * @param poses the 0-based list of poses to be used in the animation
     * @param x the initial value for the X position
     * @param y the initial value for the Y position
     * @param angle the initial value for the angle
     * @param label the name of the variable which value is to be set as label
     * @param state an hashmap of type (variable, value), representing <br/>
     * the inner state of the actor. variables <b>x</b>, <b>y</b> and <b>angle</b><br/>
     * are already defined, but can allways be redefined
     */
    public Actor(String[] figureName, /*int[] poses,*/ int x, int y, int angle, String label, HashMap<String, Object> state) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = true;
        this.label = label;
        this.state = new HashMap<String, Object>();
        this.state.put("x", x);
        this.state.put("y", y);
        this.state.put("angle", angle);
        this.state.putAll(state);
        /*if(poses != null || poses.length > 0){
            this.usedPoses = poses;
        } else {
         */   this.usedPoses = new int[] {0};
        //}
    }

    /**
     * @param figureName the anme of the images, relative to the Images directory
     * @param poses the 0-based list of poses to be used in the animation
     * @param label the name of the variable which value is to be set as label
     */
    public Actor(String[] figureName, /*int[] poses,*/ String label) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = true;
        this.label = label;
        this.state = new HashMap<String, Object>();
        this.state.put("x", 0);
        this.state.put("y", 0);
        this.state.put("angle", 0);
  /*      if(poses != null || poses.length > 0){
            this.usedPoses = poses;
        } else {
    */        this.usedPoses = new int[] {0};
       // }
    }

    /**
     * @param figureName the name of the images, relative to the Images directory
     * @param poses the 0-based list of poses to be used in the animation
     * @param x the initial value for the X position
     * @param y the initial value for the Y position
     * @param angle the initial value for the angle
     * @param label the name of the variable which value is to be set as label
     *
     */
    public Actor(String[] figureName, /*int[] poses,*/ int x, int y, int angle, String label) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = true;
        this.label = label;
        this.state = new HashMap<String, Object>();
        this.state.put("x", x);
        this.state.put("y", y);
        this.state.put("angle", angle);
/*        if(poses != null || poses.length > 0){
            this.usedPoses = poses;
        } else {*/
            this.usedPoses = new int[] {0};
        //}
        
    }


    /**
     * @param figureName the name of the images, relative to the Images directory
     * @param poses the 0-based list of poses to be used in the animation
     * @param x the initial value for the X position
     * @param y the initial value for the Y position
     * @param angle the initial value for the angle
     *
     */
    public Actor(String[] figureName, /*int[] poses,*/ int x, int y, int angle) {
        this.figuresPath = figureName;
        this.poses = new ArrayList<BufferedImage>();
        for(String fileName : this.figuresPath){
            this.poses.add(Utilities.loadImage("Images" + File.separator + fileName));
        }
        this.isLabeled = false;
        this.label = "";
        this.state = new HashMap<String, Object>();
        this.state.put("x", x);
        this.state.put("y", y);
        this.state.put("angle", angle);
/*        if(poses != null || poses.length > 0){
            this.usedPoses = poses;
        } else {*/
            this.usedPoses = new int[] {0};
        //}
    }

    private Actor(Actor aThis) {
        this.label = aThis.label;
        this.isLabeled = aThis.isLabeled;
        this.state = (HashMap<String, Object>) aThis.state.clone();
        this.poses = (ArrayList<BufferedImage>) aThis.poses.clone();
        this.figuresPath = aThis.figuresPath;
        this.usedPoses = aThis.usedPoses.clone();
        this.isAggregated = aThis.isAggregated;
    }


    
    public String[] getFiguresPath() {
        return figuresPath;
    }

    public void setFiguresPath(String[] figuresPath) {
        this.figuresPath = figuresPath;
    }

    public ArrayList<BufferedImage> getPoses() {
        return poses;
    }

    public void setPoses(ArrayList<BufferedImage> poses) {
        this.poses = poses;
    }

    public int[] getUsedPoses() {
        return usedPoses;
    }

    public void setUsedPoses(int[] usedPoses) {
        if(usedPoses != null || usedPoses.length > 0){
            this.usedPoses = usedPoses;
        } else {
            this.usedPoses = new int[] {0};
        }
    }







    public boolean isIsLabeled() {
        return isLabeled;
    }

    public void setIsLabeled(boolean isLabeled) {
        this.isLabeled = isLabeled;
    }

    public String getLabel() {
        return label;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public HashMap<String, Object> getState() {
        return state;
    }

    public void setState(HashMap<String, Object> state) {
        this.state = state;
    }



    public int getX(){
        return (Integer)state.get("x");
    }

    public int getY(){
        return (Integer)state.get("y");
    }

    public int getAngle(){
        return (Integer)state.get("angle");
    }

    public void setX(int value){
        this.state.put("x", value);
    }

    public void setY(int value){
        this.state.put("y", value);
    }

    public void setAngle(int value){
        this.state.put("angle", value);
    }

    public boolean isIsAggregated() {
        return isAggregated;
    }

    public void setIsAggregated(boolean isAggregated) {
        this.isAggregated = isAggregated;
    }

    
    public Object clone(){
        return new Actor(this);
    }


    

    /*@Override
    public String toString(){
        String s = "";
        for(int i : this.getUsedPoses()){
            s += i + " ";
        }

        return s;
    }*/

    


}
