package Alma;

import  Lisa.Parser.CTreeNode;
import  Lisa.Parser.CSyntaxTree;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Color;
import java.awt.Toolkit;

import java.util.ArrayList;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JSplitPane;
import javax.swing.JScrollPane;

import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultEdge;


public class StructDialog extends JDialog implements ActionListener {

	private Graph graph;
	private Steps stepsT;
	private TableDesign table;
	private TAB tab;
	private ArrayList vertices;
	private double zoom;
	private JButton back, forward, reset, skip, zoomM, zoomL;
	private Backend backend;
	private Motor motor;
	private CSyntaxTree root;
	private Visualize vis;
	private JSplitPane vertical;
	private boolean isConsult;
	private String fileName;

  public StructDialog(JFrame parent, 
		  CTreeNode node,
		  Object ids,
		  Hashtable types,
		  String title,
		  String file,
		  boolean isConsult1 ) 
  {
	super(parent,title,true);
        Dimension d        = Toolkit.getDefaultToolkit().getScreenSize();	
	this.stepsT          = new Steps();
	this.graph           = new Graph();
	this.table           = new TableDesign();
	this.zoom            = 1.0;
	this.backend         = new Backend();
	this.motor           = new Motor();
	this.vertices        = new ArrayList();
	this.vis             = new Visualize(this.graph);
        this.isConsult       = isConsult1;
	this.fileName	     = file;

	if (node != null)
	{
		CSyntaxTree cs       = new CSyntaxTree();
		CAlmaNode c          = new CStmtsNode((CAlmaNode)node);
		cs.setRoot(c);
		this.tab	     = new TAB((ArrayList)ids);
		this.root            = cs;
		this.backend.firstStep(this.root,true,this.tab, types, false,null,file,"Struct");
	}
	else
	{
		if (title.equals("Struct")) {
			this.tab = new TAB((ArrayList)ids);
			this.table.createTableStruct((ArrayList)ids);
		}
		else if (title.equals("Function arguments")) {
			Hashtable localInfo = (Hashtable)ids;
			TAB localIds  = (TAB)localInfo.get("arguments");
			this.tab = new TAB(localIds.getEnv());
			this.table.createTableStruct(localIds.getEnv());
		}
	}

	this.skip          = new JButton("Skip"); 
	this.reset 	   = new JButton("Reset");
	this.back 	   = new JButton("Back");
	this.forward 	   = new JButton("Forward");
	this.zoomM 	   = new JButton("Zoom +");
	this.zoomL 	   = new JButton("Zoom -");
		  
	/** local variable **/
	Table aux;

	JPanel panel           = new JPanel(new BorderLayout());
	JPanel left            = new JPanel(new BorderLayout());
	JPanel panelTable      = new JPanel(new BorderLayout());
	JPanel panelButton     = new JPanel();

	JScrollPane right, p, pEditor;
	JSplitPane horizontal;

	/************************************
	 * first steps is what we want to 
	 * visualize
	 * **********************************/
	
	// get the animations steps obtained by backend.animate
	this.stepsT = backend.getAnimationSteps();
		
	if (stepsT.getSteps()!= null)
	{
		if (stepsT.getSteps().size() > 0)
		{
			aux         = ((Table)stepsT.getSteps().get(0));
			table.createTable(aux.getTable());
			table.setCellColor(aux.getBefore(),aux.getNext());
		
			// the graph
			motor.makeGraph(aux.getAll(),vertices);
			graph.setGraph(vertices);
		}
	}
	
	right   = new JScrollPane(graph.getGraph());

	p = new JScrollPane(table.getTable());
	p.setPreferredSize(new Dimension(100,180));
	panelTable.add(new JLabel("Identifiers",JLabel.CENTER),BorderLayout.NORTH);
	panelTable.add(p,BorderLayout.CENTER);
	left.add(panelTable, BorderLayout.NORTH);
	

	/******************************************
	 * Skip button
	 * ***************************************/
	this.skip.addActionListener(new java.awt.event.ActionListener() {
         	public void actionPerformed(java.awt.event.ActionEvent e) {
		skipStepByStep();
	}
	});
		
	/******************************************
	 * Reset button
	 * ***************************************/

	reset.setEnabled(false);

	reset.addActionListener(new java.awt.event.ActionListener() {
	public void actionPerformed(ActionEvent e) {
		back.setEnabled(false);
		forward.setEnabled(true);
		stepsT.setPosition(0);

		if (stepsT.getSteps().size() > 0)
		{
			Table aux   = ((Table)stepsT.getSteps().get(0));
			vertices.clear();
			motor.makeGraph(aux.getAll(),vertices);
			graph.setGraph(vertices);
			table.setCellColor(aux.getBefore(),aux.getNext());
		}
       	}
       	});

	/******************************************
	 * Backward button
	 * ***************************************/

	back.setEnabled(false);

	back.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {		
        }
        });

	/*****************************************
	* Forward button
	* ***************************************/

	forward.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
		ArrayList ids = new ArrayList();

		backend.forwardStep();
		Table t = backend.getAnimationSteps().getLast();

		// change graph
		vertices.clear();
		motor.makeGraph(t.getAll(),vertices);
		graph.setGraph(vertices);

		// change identifiers table
		ids = t.getTable();
		if (ids.size() != 0) { 
			table.setTable(ids); 
			table.setCellColor(t.getBefore(),t.getNext());
		}
	}	
      });

	/*****************************************
	* Zoom + button
	* ***************************************/

	zoomM.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
		zoom += 0.1;
		graph.getGraph().setScale(zoom);
        }
        });

	/*****************************************
	 * Zoom + button
	 * ***************************************/

	zoomL.addActionListener(new java.awt.event.ActionListener() {
       	public void actionPerformed(java.awt.event.ActionEvent e) {
		if (zoom != 0.1)
		{
			zoom -= 0.1;
			graph.getGraph().setScale(zoom);
		}
       	}
       	});

	panelButton.add(reset);	
	panelButton.add(back);
	panelButton.add(forward);
	panelButton.add(zoomM);
	panelButton.add(zoomL);
	panelButton.add(skip);
	panel.add(panelButton,BorderLayout.CENTER);
	horizontal = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left, right);
	vertical   = new JSplitPane(JSplitPane.VERTICAL_SPLIT, horizontal, panel);

	if (this.isConsult)
	{
		this.getContentPane().add(left,BorderLayout.CENTER);
	}
	else
	{
		this.getContentPane().add(vertical);
	}
		
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	if (!this.isConsult)	{ this.setSize(750,500); }
	else {	this.setSize(360,200);	}
	horizontal.setDividerLocation(310);
	vertical.setDividerLocation((int)d.getWidth()/2-220);

        setLocation((int)d.getWidth()/2-300,(int)d.getHeight()/2-300);

	// in case of struct already initialized then we block all
	// buttons that allow us to navigate trough graph	
 	if (this.isConsult)
	{
		forward.setEnabled(false);
		back.setEnabled(false);
		reset.setEnabled(false);
		skip.setEnabled(false);
	}
		
	setVisible(true);
  }
  
 
   	public void skipStepByStep()
	{
		ArrayList vertexes = new ArrayList();
		ArrayList idents   = new ArrayList();
		Table aux;
		int last;

		if (this.stepsT.getSteps().size() > 0)
		{
			last        = stepsT.getSteps().size()-1;
			aux         = ((Table)stepsT.getSteps().get(last));
			table.setTable(aux.getTable());
			table.setCellColor(aux.getBefore(),aux.getNext());
		
			// the graph
			vertices.clear();
			motor.makeGraph(aux.getAll(),vertices);
			graph.setGraph(vertices);
		}
	}

  	public void actionPerformed(ActionEvent e) 
	{
		setVisible(false); 
		dispose(); 
  	}

    	public Table getTableIdentifiers()
	{
		Table aux1   = (Table)stepsT.getLast();	
		return aux1;
	}
	
}
 

