import Lisa.Parser.LR.LR1.*;
/*
 * Seminarska naloga pri predmetu
 *           Objektno in funkcijsko programiranje
 * 
 *      skelet za LR(1) parser oziroma template
 * 
 * Avtor : David Balazic
 */
import java.io.*;
import Lisa.Parser.*;
import Lisa.Interface.CMessage;
import Lisa.Scanner.*;
import java.util.Stack;
import java.util.zip.*;

/** CLR1Parser je glavni del LR(1) parserja. Vsebuje gradnjo stanj koncnega avtomata,
 * gradnjo krmilnih tabel ( matriki F in G ) , ter sam parser.
 * @version 0.7 <BR>
 * last change : 6-mar-98
 * @author David Balazic
 */

//public class CLR1Parser implements Parser
public class 
CParser
implements Parser
{
   // tip tokena  spada v CTokenInfo , ampak ne gre tja 
   /** spada v CTokenInfo , oznacuje da je tip tokena kategorija */
   
   public static final int cCategory = 3;  // vrednost izbrana "od oka"
   /** spada v CTokenInfo , oznacuje da je tip tokena terminal */
   public static final int cTerminal = 7;  
   
   /** interne informacije o tokenu */
   private class CTokenInfo
     {
	/** referenca na CToken */
	public CToken token;        // sam token
	/** oznaka napake */
	public boolean error;       // ce je prislo do napake
	/** LocalId ustreznega simbola */
	public int symbolId;
     }
   
   /** maksimalno stevilo stanj , ki jih vrinemo med obdelavo napake, ali stevilo
    nepopolnih produkcij, ki jih opravimo */
   int cMAX_ERRORS = 4 ;
   
   /* kode za operacije parserja */
   private final static int cOP_ERROR=-1;
   private final static int cOP_NONE=0;
   private final static int cOP_SHIFT=1;
   private final static int cOP_REDUCE=2;
   private final static int cOP_ACCEPT=3;
   
   /** krmilna tabela */
   private CLR1Table        controlTable;
   /** Podatki za reduciranje pri error recovery, 
    vsebuje stevilko najobetavnejse produkcije za vsako stanje */
   
   

   //  nekaj tabel ...
   static int errorReduceProduction[] = null;
   static int errorReduceAmount[] = null;
   static int errorReduceExpSymbol[] = null;
   static int trows = 185 , tcols = 62 ;
   static class NS { static int nextStateTableRLE[][] = null; }   static class OP { static int operationTableRLE[][] = null; }     static class PR { static int productionTableRLE[][]  = null; }   boolean isTerminalArray[] = {true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true};
   boolean termOrCatArray[] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true};
   int symbolTypeArray[] = {3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1};
   boolean symbolEpsilonArray[] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true};
   boolean symbolNullableArray[] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true,true,true,false,false,false,false,false,false,true,false,true};

   int prodLeftSideArray[] = {57,56,59,59,60,60,50,51,51,58,58,58,58,58,53,53,53,53,53,53,53,54,48,52,52,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,47,46,55,49,45};
   int prodSizeArray[] = {1,6,1,1,2,1,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,4,6,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,4,4,1};
   boolean prodIsEpsilonArray[] = {false,false,false,true,false,false,false,false,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false};
   int nrProductions = 49 ;
     
   /** izvlecek podatkov o simbolih gramatike */
   private CLR1SymbolData symbols;
   /** izvlecek podatkov o produkcijah */
   private CLR1ProductionData productions;
   
   /** localID simbola $ */
   private int eofSymbolId;
   /** localID pomoznega startnega simbola */
   private int startSymbolId;
   /** localID simbola epsilon */
   private int epsilonID;
/** stevilo redukcij opravljenih pri odpravljanju napak */
   private int errorReductionCount = 0;
   /** stevilo vstavljanj opravljenih pri odpravljanju napak */
   private int errorInsertionCount = 0;
   /** predstavlja razmerje med redukcijami in vstavljanjem v primeru napake
    */
 private float errorReductionFactor =  2; // 2:1 za vstavljanje

       static int nextStateTable[][] = null;
       static int operationTable[][] = null;
       static int productionTable[][] = null;

//   public CLR1Parser()
  //   { // konstruktor
	
   public CParser()
     {
        int nrSymbols = 62 ;
        symbols = new CLR1SymbolData(nrSymbols);
        symbols.symbols[0].symbolName = "#karel$As";
        symbols.symbols[1].symbolName = "#karel$Backisblocked";
        symbols.symbols[2].symbolName = "#karel$Backisclear";
        symbols.symbols[3].symbolName = "#karel$Beepersinbag";
        symbols.symbols[4].symbolName = "#karel$Begin";
        symbols.symbols[5].symbolName = "#karel$Beginexec";
        symbols.symbols[6].symbolName = "#karel$Beginprog";
        symbols.symbols[7].symbolName = "#karel$Defnewinst";
        symbols.symbols[8].symbolName = "#karel$Do";
        symbols.symbols[9].symbolName = "#karel$Else";
        symbols.symbols[10].symbolName = "#karel$End";
        symbols.symbols[11].symbolName = "#karel$Endexec";
        symbols.symbols[12].symbolName = "#karel$Endprog";
        symbols.symbols[13].symbolName = "#karel$Facingeast";
        symbols.symbols[14].symbolName = "#karel$Facingnorth";
        symbols.symbols[15].symbolName = "#karel$Facingsouth";
        symbols.symbols[16].symbolName = "#karel$Facingwest";
        symbols.symbols[17].symbolName = "#karel$Frontisblocked";
        symbols.symbols[18].symbolName = "#karel$Frontisclear";
        symbols.symbols[19].symbolName = "#karel$Identifier";
        symbols.symbols[20].symbolName = "#karel$If";
        symbols.symbols[21].symbolName = "#karel$Iterate";
        symbols.symbols[22].symbolName = "#karel$Leftisblocked";
        symbols.symbols[23].symbolName = "#karel$Leftisclear";
        symbols.symbols[24].symbolName = "#karel$Move";
        symbols.symbols[25].symbolName = "#karel$Nexttobeeper";
        symbols.symbols[26].symbolName = "#karel$Nobeepersinbag";
        symbols.symbols[27].symbolName = "#karel$Notfacingeast";
        symbols.symbols[28].symbolName = "#karel$Notfacingnorth";
        symbols.symbols[29].symbolName = "#karel$Notfacingsouth";
        symbols.symbols[30].symbolName = "#karel$Notfacingwest";
        symbols.symbols[31].symbolName = "#karel$Notnexttobeeper";
        symbols.symbols[32].symbolName = "#karel$Number";
        symbols.symbols[33].symbolName = "#karel$Pickbeeper";
        symbols.symbols[34].symbolName = "#karel$Putbeeper";
        symbols.symbols[35].symbolName = "#karel$Rightisblocked";
        symbols.symbols[36].symbolName = "#karel$Rightisclear";
        symbols.symbols[37].symbolName = "#karel$Then";
        symbols.symbols[38].symbolName = "#karel$Times";
        symbols.symbols[39].symbolName = "#karel$Turnleft";
        symbols.symbols[40].symbolName = "#karel$Turnoff";
        symbols.symbols[41].symbolName = "#karel$Turnon";
        symbols.symbols[42].symbolName = "#karel$While";
        symbols.symbols[43].symbolName = "$__JZTJETZJHTE32ZJEZWEEER43252543GSRETEHRJZXYXSD";
        symbols.symbols[44].symbolName = ".";
        symbols.symbols[45].symbolName = "POMOZNI_ZACETNI_SIMBOL__JZTJETZJHTEZJEZWEEERUTZGHSDFGSRETEHRJZXYXSD";
        symbols.symbols[46].symbolName = "karel$BLOCK";
        symbols.symbols[47].symbolName = "karel$CONDITION";
        symbols.symbols[48].symbolName = "karel$CONDITIONAL";
        symbols.symbols[49].symbolName = "karel$DEFINITION";
        symbols.symbols[50].symbolName = "karel$DEFSLIST";
        symbols.symbols[51].symbolName = "karel$DEFSLIST2";
        symbols.symbols[52].symbolName = "karel$ELSEPART";
        symbols.symbols[53].symbolName = "karel$INSTRUCTION";
        symbols.symbols[54].symbolName = "karel$ITERATION";
        symbols.symbols[55].symbolName = "karel$LOOP";
        symbols.symbols[56].symbolName = "karel$PROGRAM";
        symbols.symbols[57].symbolName = "karel$START";
        symbols.symbols[58].symbolName = "karel$STATEMENT";
        symbols.symbols[59].symbolName = "karel$STATSLIST";
        symbols.symbols[60].symbolName = "karel$STATSLIST2";
        symbols.symbols[61].symbolName = "epsilon";

        eofSymbolId = 43;
        startSymbolId = 45;
        epsilonID = 61;

        controlTable = new CLR1Table(trows,tcols);
	int r,c;
	for (r=0;r<trows;r++)
	    for(c=0;c<tcols;c++)
	  {
	     controlTable.tabela[r][c].nextState = new Integer(nextStateTable[r][c]);
	     controlTable.tabela[r][c].operation = operationTable[r][c];
	     controlTable.tabela[r][c].productionNumber = productionTable[r][c];
	  }
	
	int i;
	for (i=0;i<nrSymbols;i++)
	  {
	     symbols.symbols[i].isTerminal = isTerminalArray[i];
	     symbols.symbols[i].termOrCategory = termOrCatArray[i];
	     if ( symbolTypeArray[i] == 1 )
	       symbols.symbols[i].parseSymbol = new CParseSymbolTerm(symbols.symbols[i].symbolName,
							 i,symbolEpsilonArray[i],symbolNullableArray[i]);
	     else if ( symbolTypeArray[i] == 2 )
	       symbols.symbols[i].parseSymbol = new CParseSymbolNonTerm(symbols.symbols[i].symbolName,
									i,symbolNullableArray[i]);
	     else if ( symbolTypeArray[i] == 3 )
	       symbols.symbols[i].parseSymbol = new CParseSymbolCategory(symbols.symbols[i].symbolName,
									i,symbolNullableArray[i]);
	  }

	productions = new CLR1ProductionData(nrProductions);
	for  ( i=0 ; i<nrProductions ; i++)
	  {
	     productions.productions[i].leftSideID = prodLeftSideArray[i];
	     productions.productions[i].size = prodSizeArray[i];
	     productions.productions[i].isEpsilon = prodIsEpsilonArray[i];
	  }
     }
   
   /** Ali je gramatika LR(1) in ni prislo do drugih napak med inicializacijo ?<BR>
    @return <UL>
    <LI> true - parser je pripravljen za parsanje
    <LI> false - parser ni pripravljen
    </UL>
    */
   public boolean canParse()
     // ali smo zmozni parsati dano gramatiko ?
     {
	return true; 
     }
   
   /** Vrni ime parserja
    @return vrne se niz   "LR(1)" 
    */
   public String getParserName()
     /* vrni ime/tip parserja */
     {
	return "LR(1)";
     }
   
 /** parsaj vhodni tok
       @param aLexicalAnalysis lexikalno analizator za vhod
       @return sintakticno drevo ali null v primeru napake */
   public CSyntaxTree parse(LexicalAnalysis aLexicalAnalysis)
     /* razpoznaj vhodni tok tokenov in vrni sintakticno drevo */
     {
	Stack stateStack = new Stack();  // sklad stanj avtomata
	Stack treeStack = new Stack();   // sklad simbolov ( CTreeNode )
	Stack helpStack = new Stack();   // pomozni sklad ( vstavljanje CTreeNode-ov v starsevski
	//                 CTreeNode v obratnem vrstnem redu ,
	//		    kot jih dobimo iz treeStack-a ) 
	int curState;            // trenutno stanje ( vrh sklada )
	int curToken;            // trenutni token na vhodu
	int operation;           // operacija shift,reduce,accept,error
	int u;                   // stevec v zanki
	int m;                   // meja iteracij v zanki
	int prodNum;             // stevilka produkcije pri redukciji
	int leftSideID;          // LocalID leve strani produkcije
	
	boolean errormode = false ; // ali vhod ne ustreza gramatiki ?  potem se ne ukvarjamo z sint. drevesom
	int errorcount=0;        // stevilo umetno vstavljenih stanj 
	int wantedTokenId;       // token , ki ga pricakujemo - obdelava napak 
	Integer nextState;       // umetno vstavljeno stanje
	
	CTreeNode newTreeNode;   // vozlisca , ki jih bomo kreirali
	
	CTokenInfo tokenInfo = new CTokenInfo();  // informacije o tokenu
	
	CLR1Tabentry tabela[][];            // bliznjica do kontrolne tabele parserja
	tabela = controlTable.tabela;
	
	setUpCategoryIDs(aLexicalAnalysis); 
	
	CToken token =  aLexicalAnalysis.nextToken();  // poberi prvi token
	curState = 0;                                  // zacetno stanje 
	tokenToId(tokenInfo,token);    // pretvori podatke o tokenu v prirocno obliko
	
	stateStack.push(new Integer(0));               // porini na sklad prvo stanje
	
	while(true)          // "neskoncna" zanka
	  {
	     if ( tokenInfo.error )    // ce je scanner vrnil napako
	       {
		  wantedTokenId = selectWantedTokenId(curState);  // pricakovan token ...
		  if ( errorcount == 0 )  // smo na zacetku obdelave napake
		    {
		       if ( wantedTokenId != -1 )   // ne vemo kaj smo pricakovali
			 CMessage.getMessage().printError("neznan token , pricakoval : '"+
					     symbols.getParseSymbol(wantedTokenId).getName()+"'",
					     token.row(),token.column(),token.fileName());
		       else
			 CMessage.getMessage().printError("neznan token",
					     token.row(),token.column(),token.fileName());
		    }
		  
		  if ( ( token.type() == -1 ) && ( !token.isEOF() ) )
		    return null;  // ce se je scannerju zataknilo, koncamo

		  errormode=true; // vhod ni pravilen
		  token =  aLexicalAnalysis.nextToken();  // poberi naslednji token
		  tokenToId(tokenInfo,token); // pretvori podatke o tokenu v prirocno obliko
		  continue;            // in poskusi ponovno
	       }
	     curToken = tokenInfo.symbolId;      // koda tokena
	     
	     // ugotovi operacijo
	     switch ( tabela[curState][curToken].operation )
	       {
		case cOP_SHIFT :  // shift
		  errorcount=0;  // situacija spet normalna
		  stateStack.push(controlTable.destination(curState,curToken));
		  // naslednje stanje na sklad
		  curState = controlTable.destination(curState,curToken).intValue();
		  // novo stanje postane trenutno stanje
		  if ( !errormode )
		    {  // v primeru napake se ne ukvarjaj z drevesom, sicer pa
		       newTreeNode =  new CTreeNode(symbols.getParseSymbol(tokenInfo.symbolId));
		       newTreeNode.setToken(token);  // kreiramo novi TreeNode
		       treeStack.push(newTreeNode);  // in ga posljemo na sklad
		    }
		  token =  aLexicalAnalysis.nextToken();
		  tokenToId(tokenInfo,token);
		  break;
		case cOP_REDUCE : // reduce
		  errorcount=0;  // situacija spet normalna
		  prodNum = controlTable.getProductionNumber(curState,curToken);
		  leftSideID = productions.getLeftSide(prodNum);
		  
		  if (productions.isEpsilon(prodNum))
		    { // kreiraj simbol epsilon
		       CTreeNode epsTreeNode =  new CTreeNode(symbols.getParseSymbol(epsilonID));
		       epsTreeNode.setToken(null);
		       newTreeNode = new CTreeNode(symbols.getParseSymbol(leftSideID));
		       newTreeNode.addNode(epsTreeNode);
		       newTreeNode.setProdNumber(prodNum);
		    }
		  else
		    {
		       newTreeNode = new CTreeNode(symbols.getParseSymbol(leftSideID));
		       newTreeNode.setProdNumber(prodNum);
		       m = productions.getSize(prodNum);

		       if ( ! errormode ) // v primeru napake se ne ukvarjaj z drevesom
			 {
			    for (u=0;u<m;u++)
			      {
				 stateStack.pop();
				 helpStack.push(treeStack.pop());
			      }
			    for (u=0;u<m;u++)
			      newTreeNode.addNode((CTreeNode)helpStack.pop());
			 }
		       else // zgodila se je napaka , pozabi na sint. drevo
			 for (u=0;u<m;u++)
			   stateStack.pop();
		    }
		  curState=((Integer)stateStack.peek()).intValue();
		  stateStack.push( controlTable.destination(curState,leftSideID) );
		  curState = controlTable.destination(curState,leftSideID).intValue() ;
		  if ( ! errormode ) // v primeru napake se ne ukvarjaj z drevesom
		    treeStack.push(newTreeNode);
		  break;
		case cOP_ACCEPT : // accept
		  
		  if ( errormode ) // v primeru napake se ne ukvarjaj z drevesom
		    {
		       return null;
		    }
		  CSyntaxTree sTree=new CSyntaxTree();
		  sTree.setRoot (((CTreeNode)treeStack.peek()));
		  return sTree;
		case cOP_ERROR :// error
		  if (errorcount==0) // ce je do zdaj bilo OK , potem izpisi sporocilo
		    {
		       wantedTokenId = selectWantedTokenId(curState);
		       if ( wantedTokenId != -1 )
			 CMessage.getMessage().printError("napaka pri razpoznavanju, pricakoval token : '"+
					     symbols.getParseSymbol(wantedTokenId).getName()+"'",
					     token.row(),token.column(),token.fileName());
		       else
			 CMessage.getMessage().printError("napaka pri razpoznavanju",token.row(),token.column(),token.fileName());
		       errormode=true;
		    }
		  errorcount++;
		  
		  if ( errorcount <= cMAX_ERRORS)
		    {
		       if ( errorReduceProduction[curState]!=-1 && ((errorReductionCount*errorReductionFactor < errorInsertionCount) || selectNextStateOnError(curState)==null) )
			 {
			    errorReductionCount++;
			    wantedTokenId = errorReduceExpSymbol[curState];
			    prodNum = errorReduceProduction[curState];
			    leftSideID = productions.getLeftSide(prodNum);
			    if (leftSideID == startSymbolId )
			      // reduciramo na startni simbol , torej nimamo vec kaj poceti
			      return null;
			    
			    m = errorReduceAmount[curState];
			    for (u=0;u<m;u++)
			      stateStack.pop();
			    
			    curState=((Integer)stateStack.peek()).intValue();
			    stateStack.push( controlTable.destination(curState,leftSideID) );
			    curState = controlTable.destination(curState,leftSideID).intValue() ;
			    if (wantedTokenId!=-1)
			      CMessage.getMessage().printError("mozen token : '"+
						  symbols.getParseSymbol(wantedTokenId).getName()+"'",
						  token.row(),token.column(),token.fileName());
			    
			    break;			    
			 }

		       nextState=selectNextStateOnError(curState);
		       if ( nextState != null )
			 {
			    errorInsertionCount++;
			    CMessage.getMessage().printError("mozen token : '"+
				  symbols.getParseSymbol(selectWantedTokenId(curState)).getName()+"'",
				  token.row(),token.column(),token.fileName());
			    
			    stateStack.push(nextState);
			    curState=nextState.intValue();
			    break;
			 }
		    }
		  // tu ni else-ta !
		    { // konec "igre" , iscemo dolocen token , preskakujemo vhod
		       wantedTokenId = selectWantedTokenId(curState);
		       // kaj ce wanted == -1 !!!!! ???? --> pridemo do konca vhoda ...
		       while ( tokenInfo.symbolId != wantedTokenId )
			 {
			    token =  aLexicalAnalysis.nextToken();
			    tokenToId(tokenInfo,token);
			    if ( tokenInfo.symbolId == eofSymbolId )
			      return null; // prisli na konec vhoda
			 }
		       break;
		    }
		default: // sem ne smemo priti
		  CMessage.getMessage().error("interna napaka 1 , stanje : "+curState+" , token : "+token.name());
		  return null;
	       }
	  }
     }

   /** pretvori token v prirocno obliko informacije 
    @param aInfo tu se vrnejo rezultati
    @param aToken vhodni podatki
   */
   private void tokenToId(CTokenInfo aInfo,CToken aToken)
     {// pretvori token v uporabne informacije    XXXXX za optimizirat !!!!
	aInfo.token = aToken;            // postavi kazalec na CToken
	aInfo.error = false;             // do sedaj se ni napake
	if ( aToken.isEOF() && aToken.type()==-1 )
	  {  // ce konec vhoda
	     aInfo.symbolId = eofSymbolId ;
	     return;
	  }
	
	int i,        // stevec v zanki
	  type,       // tip tokena
	  symbolcount=symbols.size();  // stevilo vseh znanih simbolov
	int categoryID=-1;
	
	for (i=0 ; i<symbolcount; i++ )
	  { // primerjamo z imeni simbolov
	     if ( !symbols.getIsTerminal(i) ) //  ce neterninal -> naslednji simbol
	       continue;
	     if ( symbols.getParseSymbol(i).compareToToken(aToken) )
	       {
		  if (symbols.getTermOrCategory(i) )
		    { // navaden terminal -> return
		       aInfo.symbolId = i; // bnf.elementAt(i).getLocalId()   ==   i    !!!
		       return;
		    }
		  else
		    { // kategorija , pogledamo se ce ustreza kaksnemu terminalu
		       // preveri ali je ta vhodni simbol smiseln
		       categoryID = i;
		    }
	       }
	  }
	aInfo.symbolId = categoryID;
	if ( categoryID != -1 )
	  return; // nasli kategorijo -> vrni OK
	// sicer napaka
	aInfo.error = true; 
	return;
     }
/**  postavi categoryID za CParseSymbol-e
 @param aLexicalAnalysis lexikalni analizator */  
   void setUpCategoryIDs(LexicalAnalysis aLexicalAnalysis)
     { // postavi categoryID za CParseSymbol-e
	for ( int i = 0 ; i < symbols.symbols.length ; i++ )
	  {
	     if ( symbols.getIsTerminal(i) && !symbols.getTermOrCategory(i) )
	       { // dobili smo kategorijo ...
		  ((CParseSymbolCategory)symbols.getParseSymbol(i)).setCategoryId
		    ( aLexicalAnalysis.getCategoryId(symbols.getName(i).substring(1,
						       symbols.getName(i).length()  )));
	       }
	  }
     }
   
   /** izbere naslednje stanje v primeru napake.
    Izbere se stanje v katerega gre najvec prehodov iz trenutnega. 
    @param aCurState trenutno stanje parserja
    @return stevilka naslednjega stanja
    */
   Integer selectNextStateOnError(int aCurState)
     {
	int nextStates[] = new int [controlTable.rows];  // int za vsa stanja
	int i;  // stevec zanke
	
	for ( i=0 ; i<controlTable.rows ; i++)
	  nextStates[i]=0;   // brisi polje
	
	int symbolMax = symbols.size();  // stevilo simbolov
	
	for (i=0 ; i<symbolMax ; i++ )  // skozi vse simbole
	  if (controlTable.operation(aCurState,i)== cOP_SHIFT)  // ce obstaja prehod pri tem simbolu
	    nextStates[controlTable.destination(aCurState,i).intValue()]++;  // belezimo prehod v neko stanje
	
	int nextState=-1;
	int max = 0;
	
	// poiscemo stanje v katerega gre najvec prehodov
	for ( i=0 ; i<controlTable.rows ; i++ )
	  if ( nextStates[i] > max ) {
	     max = nextStates[i];
	     nextState = i;
	  }
	if ( nextState == -1 )
	  return null;   // nismo nasli nobenega stanja
	
	return new Integer(nextState);  // vrni stevilko stanja
     }
   
   
 /** ugotovi pricakovan token za trenutno stanje
  @param aCurState trenutno stanje parserja
  @return id pricakovanega tokena
  */
   int selectWantedTokenId(int aCurState)
     {
	int nextStates[] = new int [controlTable.rows]; // stevilo prehodov v doloceno stanje
	int i;    // stevec v zanki
	
	// inicializiraj polje , postavi elemente na 0
	for ( i=0 ; i<controlTable.rows ; i++)
	  nextStates[i]=0;
	
	int symbolMax = symbols.size(); // stevilo simbolov
	
	// prestej prehode v vsako stanje
	for (i=0 ; i<symbolMax ; i++ )
	  {
	     if (controlTable.operation(aCurState,i) == cOP_SHIFT)
	       nextStates[controlTable.destination(aCurState,i).intValue()]++;
	  }
	
	// poisci stanje v katerega se gre najvec krat
	int nextState=-1;  //  stanje v katerega gre najvec prehodov
	int max = 0;  // trenutni maximum stevila prehodov
	for ( i=0 ; i<controlTable.rows ; i++ )
	  if ( nextStates[i] > max ) {
	     max = nextStates[i];
	     nextState = i;
	  }
	// ni nobenega nasledenjga stanja , vrni -1
	if ( nextState == -1 )
	  return -1;
	
	// poisci in vrni prvi simbol , ki povzroci prehod v naj... stanje
	for (i=0 ; i<symbolMax ; i++ )
	  if (controlTable.operation(aCurState,i) == cOP_SHIFT)
	    if ( controlTable.destination(aCurState,i).intValue() == nextState)
	      return i;
	return -1;
     }
     
     private static void decompress(int[][] result, int[][] RLE)
      {
        for (int i = 0; i<RLE.length; i++)
	 {
	   int a = 0;
	   for (int j = 0; j<RLE[i].length; j+=2)
	    {
	      for (int k =0; k<RLE[i][j]; k++)
	        result[i][a++] = RLE[i][j+1];
	    }
	 }
      } 

static class EC0 { static byte[] encodedString={31,-117,8,0,0,0,0,0,0,0,-19,-99,-41,-105,29,-59,17,-58,103,-109,-124,-92,85,14,43,33,9,73,72,40,32,105,113,-64,1,25,-29,28,4,-58,56,39,57,-32,-100,3,-40,-62,56,1,-50,-58,9,103,112,-114,56,-30,-124,-93,-4,-9,-8,-55,-25,-8,-39,-113,-72,70,59,-115,74,-83,-23,-102,-82,123,-69,103,-70,123,-66,-17,-100,58,-77,-37,93,85,-35,-9,-73,61,61,51,-91,123,-81,30,-6,111,-75,112,-10,-10,106,-10,-52,-23,-101,-1,117,-21,-31,59,-2,-13,-16,-125,-77,85,117,-25,7,-86,-86,58,71,-10,24,-78,-103,106,69,-26,-25,85,-51,-15,26,-78,-43,100,39,-55,-74,-111,-19,32,-37,78,-74,-127,108,19,-39,102,-78,-115,100,91,-56,-74,-110,45,-109,-83,33,91,36,91,79,-74,-106,108,29,-39,66,-109,111,-74,-55,25,34,-33,-55,38,-33,126,-78,125,100,7,-55,-114,-111,93,69,118,-124,-20,56,-39,-27,100,-69,-56,-10,-112,-19,38,59,64,118,-120,-20,106,-78,-61,100,71,-55,78,-112,93,73,118,5,-39,94,54,-89,-48,121,-105,27,6,-13,-116,-127,25,107,-71,-7,125,-34,106,-101,9,-64,-54,53,-42,52,57,-19,92,-4,-9,-82,62,-97,-74,-91,-26,119,-33,54,-23,-72,116,-10,-74,-22,-82,-86,-42,-71,106,69,51,-52,102,-101,-29,-100,-11,-5,-76,54,31,56,95,-88,60,109,121,99,-28,-82,115,46,52,-57,57,54,-50,-84,-43,54,-57,108,85,-128,113,-25,-83,-100,115,-127,114,26,-77,95,-121,-44,-73,-48,50,-105,-74,54,-77,-29,-14,92,109,109,-100,-23,2,-13,49,-57,-13,-29,94,-76,-38,-81,35,123,-36,35,-115,-22,-88,-26,-8,20,-78,-83,-26,-25,-26,88,-17,19,-5,31,113,-88,-18,23,-6,-42,53,-57,42,80,-66,-75,-82,-66,105,84,-83,-20,-31,-121,35,-28,-67,-116,108,-79,-7,-7,84,-75,-78,79,-98,31,-85,-18,99,126,-89,-116,53,-65,79,-51,-54,-50,59,109,78,106,63,-63,126,-66,-24,117,116,-12,93,-37,50,-105,-74,-74,83,45,-29,92,-46,-42,-4,110,-30,-81,101,62,-89,-72,15,-35,-42,-52,-99,57,115,122,-23,127,-1,-66,101,-41,125,-9,94,-1,-24,125,-51,-93,103,-63,13,2,-127,25,87,95,78,106,-50,-5,57,-69,-67,-101,64,110,106,118,-63,5,95,-1,-14,8,104,5,2,50,-127,-6,-70,-39,-9,-116,-6,-106,-33,26,-96,-2,-43,125,-51,104,26,-15,-21,-87,-81,-36,4,-88,109,-51,20,51,49,79,67,-117,-84,109,-67,71,92,-3,-76,-75,-79,-91,-35,60,-127,109,49,119,103,-84,111,91,115,-36,-34,18,103,-98,116,118,-78,-74,-6,-23,-76,126,74,-35,93,-1,30,103,31,-96,124,123,66,-26,-117,41,-20,-124,-30,89,-80,119,-46,-84,-43,74,125,99,31,-65,-41,-92,-97,15,120,-60,-43,-75,-111,-125,45,-19,-121,-40,125,-27,17,-85,-17,104,115,60,-42,18,87,-41,94,-114,91,-9,-107,117,93,97,-39,-36,-21,98,13,92,76,-96,90,-87,-9,61,-74,98,-49,-120,-110,-56,-17,-15,-51,-67,-8,19,-56,-98,72,-10,36,-78,39,91,62,-41,-79,123,-11,-6,57,-13,122,-78,-89,-42,99,-110,61,-115,-20,-23,100,-49,-80,98,-98,73,-10,44,-33,87,65,-66,-49,-10,-11,-75,-107,-50,26,-96,57,60,103,-120,113,-45,33,48,-108,64,0,4,64,0,4,64,-32,-110,-5,-127,-25,-110,61,-113,-20,-7,62,-47,-28,119,-102,-20,70,-78,-101,-56,94,64,118,51,-39,11,45,-97,91,-56,94,-44,-4,-4,98,-78,-105,-112,-67,-108,-20,101,100,47,39,123,5,-39,43,-83,-104,87,-111,-67,-38,-9,85,-112,-17,107,124,125,109,97,13,-128,0,8,-128,0,8,-128,-64,100,4,-56,-1,76,-84,25,-11,-83,81,86,74,95,-53,125,46,16,32,123,-99,-33,-21,43,75,-93,-83,19,-66,-66,66,-99,-80,81,49,117,-62,55,-8,-6,-38,74,103,13,-48,28,110,29,98,-36,116,8,12,37,16,0,1,16,0,1,16,40,-90,78,-8,70,95,95,91,88,3,32,0,2,32,0,2,32,-96,39,64,-66,111,-118,57,-93,-66,53,-54,26,-47,-101,-71,15,-50,2,16,0,1,16,0,1,16,0,1,16,0,1,16,0,1,16,0,1,16,0,1,16,0,1,16,0,1,16,72,-101,0,-51,-21,45,-79,-57,72,-101,64,45,-102,-37,91,99,-26,79,-97,64,108,-127,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,-128,63,1,-14,121,91,31,51,-22,91,-109,-81,1,-118,121,123,-116,25,-11,45,-100,5,-30,123,-53,-33,49,105,86,-118,125,39,-39,-69,-56,-34,-51,-38,-34,-29,17,-9,94,-78,-9,-75,-76,-65,-97,-84,-2,-82,-7,-37,-56,110,-73,-6,62,-40,28,63,-44,18,119,-106,-20,14,-78,15,-77,-74,59,-71,-49,68,-97,47,-8,-120,-81,111,14,74,-5,44,-96,121,125,52,-10,24,105,19,-88,69,115,-5,88,-52,-4,-86,-85,-31,-57,99,-50,100,40,-91,-65,6,98,11,4,-60,-85,-31,39,38,-51,74,-79,117,-42,-69,-55,-18,97,109,-97,-12,-120,-5,20,-39,-89,91,-38,63,67,-10,89,-78,-49,-111,125,-34,-22,-5,66,115,-68,-73,37,-18,-117,100,95,34,-5,50,107,-5,10,-9,25,-27,119,82,125,-107,-5,-116,-110,-64,125,-36,7,-5,-64,40,63,113,89,127,43,-41,-41,42,124,43,87,-93,98,-66,-107,-21,-21,-66,-66,-74,-46,89,3,52,-121,111,12,49,110,58,4,-122,18,8,-128,0,8,-128,0,8,20,-13,-83,92,-33,-12,-11,-75,-123,53,0,2,32,0,2,32,0,2,32,48,-54,90,-15,-73,-72,-49,40,-85,100,-33,-26,62,-93,36,-16,29,-18,51,74,2,117,-91,-12,-2,10,-107,-46,70,-59,84,74,31,-16,-11,-75,-107,-50,26,-96,57,124,119,-120,113,-45,33,48,-108,64,0,4,64,0,4,64,-96,-104,74,-23,-9,124,125,109,97,13,-128,0,8,-128,0,8,-28,75,-128,-26,-4,-3,16,121,-14,37,16,74,32,48,-47,103,76,126,16,115,70,125,43,-19,53,64,-13,-6,97,-20,49,-46,38,80,-117,-26,-10,-93,-104,-7,85,-97,49,-7,113,-52,-103,12,-91,-12,-41,-128,75,52,-25,-97,-124,-56,-109,47,-127,80,2,-127,-119,-82,-122,63,-115,57,-93,-66,-107,-10,26,-96,121,-3,44,-10,24,105,19,-88,69,115,-5,121,-52,-4,-86,-85,-31,47,98,-50,100,40,-23,-42,0,-7,61,24,123,70,-95,68,115,-3,-91,-113,95,-6,103,65,108,-115,-14,93,52,-65,-30,62,-93,-4,6,-122,95,115,-97,81,18,-8,13,-9,81,-17,-124,-65,-11,-15,75,65,52,-41,-33,-7,-8,97,39,28,-27,78,-8,16,-9,25,37,-127,-33,115,-97,81,18,-8,3,-9,25,-27,-89,-17,-1,-56,125,-80,19,-26,75,-128,-26,-4,-89,16,121,-14,37,16,74,32,48,-54,119,86,-1,-103,-5,96,13,-28,75,-128,-26,-4,112,-120,60,-7,18,8,37,16,0,1,117,125,-32,47,-79,103,20,74,52,-41,-65,-6,-8,97,13,-88,-41,-64,-33,98,-49,40,-108,104,-82,127,-9,-15,27,101,-99,-16,31,-36,7,103,-63,40,43,36,-1,-28,62,88,3,32,0,2,-25,9,-52,19,-127,115,-35,44,-88,111,-90,-49,-71,-59,82,-3,58,-38,94,-117,76,-128,-38,103,-7,49,7,-71,94,-87,75,121,-98,15,33,-1,34,-39,18,-104,11,-107,-85,-13,44,40,98,15,-112,-28,-67,15,100,65,98,-110,121,-118,119,-121,-50,108,93,-25,-95,-39,-115,120,14,-97,-39,-71,118,49,-98,-49,-18,55,-65,119,-59,-71,-38,-30,-20,3,-71,-84,-103,90,-27,93,13,-75,-46,-97,5,62,52,-54,56,11,-92,87,-22,-79,15,-52,26,-13,-115,-31,113,82,-66,-48,-21,-15,-110,-49,96,-73,114,22,102,60,-45,102,109,62,-110,-65,20,-29,59,15,95,95,91,-23,-36,17,13,-75,123,-30,44,0,1,16,0,1,16,0,1,-36,15,96,13,-128,0,8,-128,0,8,-128,-64,100,-49,-122,-45,92,125,82,-45,56,43,-91,-36,-25,2,-127,-110,-2,-82,26,-95,78,-88,-33,9,53,59,90,126,59,-95,-117,-77,48,-29,-126,-97,11,-6,-42,80,-5,16,-50,2,16,0,1,16,0,1,16,-64,-3,0,-42,0,8,-128,0,8,-128,0,8,76,-12,77,-123,69,-43,-110,-58,89,35,-30,62,-23,-44,7,-38,20,122,-59,-73,9,4,64,0,4,64,0,4,64,0,4,64,0,4,64,0,4,64,0,4,64,0,4,64,0,4,64,0,4,-110,39,16,-67,46,-99,54,-127,90,-79,41,-88,-2,79,-105,-24,43,114,8,-127,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,0,8,-128,-128,-118,64,81,-17,39,54,-102,-68,86,92,10,17,124,11,-57,100,-33,65,-47,-91,-86,-24,-9,-106,-105,-78,-6,-115,-46,-2,55,-93,62,104,-89,77,-96,86,108,10,-72,26,-30,90,-128,-49,-102,-31,106,8,2,32,-112,-2,-43,-80,77,33,-9,-126,113,126,-30,-110,-73,-31,127,112,-64,-73,112,-92,-77,15,76,-13,42,-90,17,-50,2,16,0,1,16,0,1,16,-64,-3,0,-42,0,8,-128,0,8,-128,0,8,-96,86,-116,58,33,8,-128,0,42,-91,-23,84,-55,-76,10,-75,35,-30,-55,40,-99,53,48,-51,-85,-104,70,-23,16,-48,42,-50,89,-112,-109,64,0,4,64,0,4,112,63,96,-5,-5,-6,-38,-62,26,0,1,16,0,1,16,-56,-102,64,-112,39,41,-68,-81,24,4,-16,25,-109,-76,-9,-127,62,104,-89,77,-96,86,108,10,-8,-116,73,-6,107,-64,-91,-31,-82,-122,-102,61,61,-17,107,1,8,8,-77,44,106,63,72,123,31,-64,-43,-80,22,-82,-122,-79,-91,-37,9,-85,11,-11,-66,-28,-97,11,124,-1,98,120,31,17,-34,65,1,2,32,-96,-33,9,115,-71,39,12,-77,19,-90,-84,80,-69,51,-50,2,16,0,1,16,64,-75,60,-21,107,65,-28,42,89,51,-54,-120,-97,11,106,-115,-101,0,118,66,-41,-33,93,-77,-93,97,39,-116,-87,126,118,-62,-108,21,106,45,-128,0,-18,-120,116,87,-61,80,-25,94,31,-14,-99,43,-82,5,-70,125,32,-91,53,-32,113,38,78,-65,6,70,126,71,-44,120,-29,-82,120,34,-107,-77,6,82,86,63,119,68,-29,62,11,82,87,-48,53,48,79,-39,-50,-27,-53,34,-108,58,-49,-121,-53,-8,-79,68,-27,-71,6,-22,9,-121,-54,-107,39,-127,-112,2,-127,-50,125,96,85,-33,51,-22,91,-109,-83,1,-14,-97,-117,53,-93,-66,-123,-77,-96,-13,44,88,-51,-113,37,106,-94,-9,-41,22,115,6,-44,18,-97,18,-73,-71,-94,-92,62,-45,111,-52,55,-122,-57,73,-7,124,-14,104,-124,125,0,4,-60,-77,96,-125,43,74,-22,51,-3,-58,124,99,120,-100,-108,-49,39,-113,70,34,-127,77,-62,76,-99,125,-90,-33,-104,111,12,-113,-109,-14,-7,-28,-47,72,36,-80,89,-104,-87,-77,-49,-12,27,-13,-115,-31,113,82,62,-97,60,26,-119,4,54,10,51,117,-10,-103,126,99,-66,49,60,78,-54,-25,-109,71,35,-111,-64,22,97,-90,-50,62,-45,111,-52,55,-122,-57,73,-7,124,-14,104,36,18,-40,42,-52,-44,-39,103,-6,-115,-7,-58,-16,56,41,-97,79,30,-115,112,53,20,-41,-64,26,87,-108,-44,103,-6,-115,-7,-58,-16,56,41,-97,79,30,-115,68,2,-117,-62,76,-99,125,-90,-33,-104,111,12,-113,-109,-14,-7,-28,-47,72,36,-80,94,-104,-87,-77,-49,-12,27,-13,-115,-31,113,82,62,-97,60,26,-119,4,-42,10,51,117,-10,-103,126,99,-66,49,60,78,-54,-25,-109,71,35,-111,-64,58,97,-90,-50,62,-45,111,-52,55,-122,-57,73,-7,124,-14,104,36,18,88,16,102,-22,-20,51,-3,-58,124,99,120,-100,-108,-49,39,-113,70,-72,26,78,92,41,45,-26,-33,84,-79,6,80,39,-44,-41,9,93,-19,-74,79,-123,58,97,38,2,1,125,-99,-48,-43,110,-5,84,-59,-42,9,93,-19,-74,79,85,108,-99,-48,-43,110,-5,84,-59,-42,9,93,-19,-74,79,85,108,-99,-48,-43,110,-5,84,-59,-42,9,93,-19,-74,79,-123,58,97,38,-46,-41,9,93,-19,-74,79,85,108,-99,-48,-43,110,-5,84,-59,-42,9,93,-19,-74,79,85,108,-99,-48,-43,110,-5,84,-59,-42,9,93,-19,-74,79,85,108,-99,-48,-43,110,-5,84,-88,19,102,-94,-119,106,68,-59,-44,8,107,-91,-67,6,104,94,-5,99,-113,-111,60,-127,125,-79,-57,72,-98,-64,-63,-40,99,36,79,-32,88,-20,49,-110,39,112,85,-20,49,-110,39,112,36,-10,24,-55,19,56,30,123,-116,-28,9,92,30,123,-116,-28,9,-20,-118,61,70,-14,4,-10,-60,30,35,121,2,-69,99,-113,-111,60,-127,3,-79,-57,72,-98,-64,-95,-40,99,36,79,-32,-22,-40,99,36,79,-32,112,-20,49,-110,39,112,52,-10,24,-55,19,56,17,123,-116,-28,9,92,25,123,-116,-28,9,92,17,123,-116,-28,9,-20,-115,61,70,-38,4,-6,16,8,-88,-2,79,-105,-24,117,-37,33,-92,34,-80,-81,-121,9,-11,46,21,-127,-24,117,-37,33,-92,34,16,-67,110,59,-124,84,4,-94,-41,109,-121,-112,-118,64,-12,-70,-19,16,82,17,-120,94,-73,29,66,42,2,-47,-21,-74,67,72,69,32,122,-35,118,8,-87,8,68,-81,-37,14,33,21,-127,-24,117,-37,33,-92,34,16,-67,110,59,-124,84,4,-94,-41,109,-121,-112,-118,64,-12,-70,-19,16,82,17,-120,94,-73,29,66,42,2,-47,-21,-74,67,72,69,32,122,-35,118,8,-87,8,68,-81,-37,14,33,21,-127,-24,117,-37,33,-92,34,16,-67,110,59,-124,80,41,5,1,-15,-35,-11,-13,-82,40,-87,-49,-12,27,-13,-115,-31,113,82,62,-97,60,26,97,13,-128,0,8,-128,0,8,-120,-41,-126,-109,-82,40,-87,-49,-12,27,-13,-115,-31,113,82,62,-97,60,26,-23,-81,-122,-102,-85,26,-82,-122,57,8,4,64,32,79,2,85,-64,-1,109,15,-33,-54,-43,-7,63,56,108,-29,-57,18,-107,-25,89,16,82,32,-48,121,22,108,-32,-57,18,-43,73,96,19,63,-106,-88,78,2,-101,-7,-79,68,117,18,-40,-56,-113,37,-86,-109,-64,22,126,44,81,-99,4,-74,-14,99,-119,-62,-43,-80,115,13,-84,-31,-57,18,-43,73,96,-111,31,75,84,39,-127,-11,-4,88,-94,58,9,-84,-27,-57,18,-43,73,96,29,63,-106,-88,78,2,-41,-16,99,-119,-46,87,-53,53,85,-17,-68,-85,-27,99,17,8,-128,0,42,-91,-7,-82,-127,80,-43,-37,124,9,-124,18,8,-28,75,32,84,-11,54,107,2,65,-86,-73,89,19,8,82,-67,-51,-102,64,-112,-22,109,-42,4,-126,84,111,-77,38,16,-92,122,-101,47,-127,80,-54,-105,64,-88,-22,109,-42,4,-126,84,111,-77,38,16,-92,122,-101,53,-127,32,-43,-37,-84,9,4,-87,-34,-26,75,32,-108,-60,74,-23,118,87,-108,-44,103,-6,-115,-7,-58,-16,56,41,-97,79,30,-115,68,2,-53,-62,76,-99,125,-90,-33,-104,111,12,-113,-109,-14,-7,-28,-47,8,103,1,8,-128,0,8,-128,-128,-2,106,-88,-71,-86,-107,121,53,-44,92,-43,112,53,-52,65,32,0,2,32,32,-18,-124,59,93,81,82,-97,-23,55,-26,27,-61,-29,-92,124,62,121,52,-22,124,31,-47,73,126,44,81,56,11,64,0,4,-12,59,-95,102,71,-53,127,39,76,89,-95,118,-25,124,9,-124,18,8,-128,0,8,-120,-41,-126,29,-82,40,-87,-49,-12,27,-13,-115,-31,113,82,62,-97,60,26,97,13,116,62,23,108,-25,-57,18,-43,73,96,-103,31,75,20,-50,2,-3,78,-88,-39,-47,-80,19,-26,-96,124,9,-124,-38,-99,-77,38,16,100,119,22,-9,-127,37,97,116,103,-97,-23,55,-26,27,-61,-29,-92,124,62,121,52,-22,-68,26,-18,-28,-57,18,-91,95,3,-102,-65,101,-2,107,32,101,-123,90,-105,-7,18,8,-91,-50,125,96,7,63,-106,40,-84,-127,124,9,-124,90,-105,-99,103,-63,18,63,-106,-88,-84,-41,64,-112,-65,-54,-1,1,25,-23,15,-9,93,56,2,0};};

static class EC { static byte[][] encodedString = {EC0.encodedString}; static int length=3418;};

     static {
       if (nextStateTable!=null) {
       nextStateTable = new int[trows][tcols];
       operationTable = new int[trows][tcols];
       productionTable = new int[trows][tcols];
       decompress(nextStateTable, NS.nextStateTableRLE);
       decompress(operationTable, OP.operationTableRLE);
       decompress(productionTable, PR.productionTableRLE);
       } else {
         try {

           InputStream res = null;
           if (EC.encodedString==null)
		      res = CTokenInfo.class.getResourceAsStream("LR1Tables.dat");
		    else {
		      byte[] data = new byte[EC.length];
		      int pos = 0;
		      for (int i = 0; i<EC.encodedString.length; i++) {
		      	System.arraycopy(EC.encodedString[i],0,data,pos,EC.encodedString[i].length);
		      	pos +=EC.encodedString[i].length;
		      }
		      res = new ByteArrayInputStream(data);
		    }
			ObjectInputStream ois = new ObjectInputStream(new GZIPInputStream(res));
			errorReduceProduction = (int[])ois.readObject();
			errorReduceAmount = (int[])ois.readObject();
            errorReduceExpSymbol = (int[])ois.readObject();
			nextStateTable = (int[][])ois.readObject();           
			operationTable = (int[][])ois.readObject();
			productionTable = (int[][])ois.readObject();
         } catch (Exception e) {
           e.printStackTrace();
         }
       }
     }
}
// to je zadnja vrstica template-ja
