package Alma;

import java.awt.Point;
import Lisa.Parser.CTreeNode;
import Lisa.Interface.Component.SyntaxTree.CSyntaxTreeSetup;
import org.jgraph.graph.DefaultGraphCell;

public class CParentNode {
    protected CSyntaxTreeSetup ssetup;	
    private Object  label;
    private CParentNode parent, child, sibling;
    private boolean hiden, evaluated, animated;
    private int width, height, border, parent_dist, level, row, column;
    private Point position, offset;
    private CPolygon contour;
    private DefaultGraphCell vertice;
    // used in each step of rewritten
    private Object value;

   /**
    * @param l label
    * @param p parent
    * @param c child
    * @param s sibling
    * @param hiden
    * @param vertice
    */
   public CParentNode(Object l, CParentNode p, CParentNode c, CParentNode s, boolean hiden, DefaultGraphCell vertice) {
	String name;
	CTreeNode aux;

	this.label       = l;
	this.parent      = p;
	this.child       = c;
	this.sibling     = s;
	this.position    = new Point(0,0);
	this.offset      = new Point(0, 0);	
	this.hiden       = hiden;
	this.vertice     = vertice;
	this.evaluated   = false;
    this.animated    = false;
	this.level       = -1;	// have not initialized
	this.value       = null;
	this.ssetup      = new CSyntaxTreeSetup();
	
	aux              = (CTreeNode)l;
	name             = aux.getSymbol().getName();
	
	if (aux.getToken()!=null) {
		this.row         = aux.getToken().row();
		this.column      = aux.getToken().column();
	}
	else {
		this.row	 = -1;
		this.column	 = -1;
	}

	if (name.equals("ASSIGN") || name.equals("OPER")) {
		this.width       = 120;
		this.height      = 35;	
	}
	else if (name.equals("CONST")) 	{
		this.width       = 120;
		this.height      = 35;
	}
	else {
		this.width       = 120;
		this.height      = 35;	
	}

	this.border      = 10;
	this.parent_dist = 30;
        contour          = new CPolygon();
    }

    public CParentNode(Object l, CParentNode p, CParentNode c, CParentNode s, boolean hiden, Point position, Point offset, 
		    int width, int height, int level, CPolygon contour,DefaultGraphCell vertice, Object value, boolean evaluated, boolean animated) {
	this.label       = l;
	this.parent      = p;
	this.child       = c;
	this.sibling     = s;
	this.position    = position;
	this.offset      = offset;	
	this.hiden       = hiden;
	this.vertice     = vertice;
	this.level       = level;
	this.width       = width;
	this.height      = height;	
    this.contour     = contour;
	this.vertice     = vertice;
	this.evaluated   = evaluated;
	this.value       = value;
    this.animated    = animated;
    }
    
   public void setSetup( CSyntaxTreeSetup s)
   {
	ssetup =  s;
   }
   
   public CSyntaxTreeSetup getSetup()
   {
	return ssetup;
  }

   public synchronized void calculateTreeDimension() {
      ssetup.picX1=999999;
      ssetup.picY1=999999;
      ssetup.picX2=-999999;
      ssetup.picY2=-999999;
      calculateIt(this);
      ssetup.picWidth=ssetup.picX2-ssetup.picX1+20;
      ssetup.picHeight=ssetup.picY2-ssetup.picY1+30;
      ssetup.rootY=this.position.y-ssetup.picY1+10;
   }
   
   private void calculateIt(CParentNode t) {
	  if (t != null) {
		 if (t.position.x<ssetup.picX1) ssetup.picX1=t.position.x;
		 if (t.position.y<ssetup.picY1) ssetup.picY1=t.position.y;
		 if (t.position.x+t.width>ssetup.picX2) ssetup.picX2=t.position.x+t.width;
		 if (t.position.y+t.height>ssetup.picY2) ssetup.picY2=t.position.y+t.height;
		 if (!t.hiden) calculateIt(t.child);
		 calculateIt(t.sibling);
	  }
   }
   public void copyLevels(CParentNode t)
   {
	   if (t!=null)
	   {
		this.setLevel(t.getLevel());
	     	copyLevels(t.getChild());
		copyLevels(t.getSibling());
	   }
   }
   
   public void printLevels()
   {
	   if (this!=null)
	   {
		System.out.println(this.getLevel());
	     	if(this.getChild()!=null) this.getChild().printLevels();
		if(this.getSibling()!=null) this.getSibling().printLevels();
	   }
   }
   
   public int getParentDist()
   {
	   return this.parent_dist;
   }

   public int getBorder()
   {
	   return this.border;
   }
   
   public Object getLabel()
   {
	   return this.label;
   }

   public CParentNode getParent()
   {
	   return this.parent;
   }
   
   public CParentNode getChild()
   {
	   return this.child;
   }
    
   public CParentNode getSibling()
   {
	   return this.sibling;
   }

   public boolean getHiden()
   {
	   return this.hiden;
   }

    public boolean isAnimated() {
        return animated;
    }

  

   public Point getPosition()
   {
	   return this.position;
   }

   public int getWidth()
   {
	   return this.width;
   }

   public int getHeight()
   {
	   return this.height;
   }


   public DefaultGraphCell getVertice()
   {
	   return this.vertice;
   }
   
   public int getLevel()
   {
	   return this.level;
   }

   public boolean Evaluated()
   {
	   return this.evaluated;
   }

   public Object getValue()
   {
	   return this.value;
   }  

   public int getRow()
   {
	   return this.row;
   }  

   public int getColumn()
   {
	   return this.column;
   }  

   /** set attributes
    * */
   
   public void setLabel(Object l)
   {
	   this.label = l;
   }

   public void setParent(CParentNode p)
   {
	   this.parent = p;
   }

   public void setChild(CParentNode c)
   {
	   this.child = c;
   }
    
   public void setSibling(CParentNode s)
   {
	   this.sibling = s;
   }

   public void setHiden(boolean hiden)
   {
	   this.hiden = hiden;
   }

   public void setAnimated(boolean animated) {
        this.animated = animated;
    }

   public void setPosX(int x)
   {
	   this.position.x = x;
   }

   public void setPosY(int y)
   {
	   this.position.y = y;
   }

   public void setLevel(int level)
   {
	   this.level = level;
   }

   public void setVertice(DefaultGraphCell vertice)
   {
	   this.vertice = vertice;
   }
   
   public void setEvaluated(boolean evaluated)
   {
	  this.evaluated = evaluated;
   }

   public void setValue(Object value)
   {
	  this.value = value;
   }

   /**
    * Returns true if current node don't have childs -- is leaf
    * */

   public boolean isTerminal()
   {
	   boolean res = false;

	   if (this.label!=null)
	   {
		   CTreeNode node = (CTreeNode)this.label;
		   res = (node.getNodes().length == 0);
	   }

	   return res;
   }

   /**************************************
    *
    * returns true if a node have child
    * 
    * ***********************************/
   
   public boolean haveChild()
   {
	   return (this.child != null);
   }
   
    /**************************************
    *
    * returns true if a child of a node
    * have a sibling
    * 
    * ***********************************/
   
   public boolean haveChildSibling()
   {
	   boolean res = false;
	   
	   if (this.child != null)
	   {
		   res = (this.child.getSibling() != null);
	   }

	   return res;
   }

   /**************************************
    *
    * returns true if a child of a node
    * have a second sibling
    * 
    *
    *	   child
    *	  /
    * node - sibling1
    *    \ 
    *     sibiling2
    * ***********************************/
   
   public boolean haveSecondSibling()
   {
	   boolean res = false;
	   
	   if (this.haveChildSibling())  {
		   res = (this.child.getSibling().getSibling() != null);
	   }

	   return res;
   }

   /***************************************
    *
    * test if current node is evaluated 
    * and not null
    *
    * *************************************/
   
   public boolean isEvaluatedChild()
   {
	   boolean flag = true;

	   if (this != null)
	   {
		   if (this.child != null)
		   {
			   flag = this.child.Evaluated();
		   }
	   }
	   return flag;
   }

   public boolean isEvaluatedSibling1()
   {
	   boolean flag = true;

	   if (this.child != null)
	   {
		   if (this.child.getSibling() != null)
		   {
		   	flag = this.child.getSibling().Evaluated();
		   }
	   }
	   return flag;
   }
  
   public boolean isEvaluatedSibling2()
   {
	   boolean flag = true;

	   if (this.child != null)
	   {
		   if (this.child.getSibling() != null)
		   {
			   if (this.child.getSibling().getSibling() != null)
			   {
				   flag = this.child.getSibling().getSibling().Evaluated();
			   }
		   }
	   }
	   return flag;
   }
   
   /***************************************
    *
    * evaluate a subtree (change variable 
    * "evaluated" to true)
    *
    * *************************************/

   public void setEvaluateSubTree(boolean e)
   {
	if (this!=null)
	{
		this.evaluated = e;
		if (this.child!=null)
		{
			this.child.setEvaluateSubTree(e);

			if (this.child.getSibling() != null)
			{
				this.child.getSibling().setEvaluateSubTree(e);

				if (this.child.getSibling().getSibling()!=null)
					this.child.getSibling().getSibling().setEvaluateSubTree(e);
			}
		}			
	}
   }

   /***************************************
    *
    * hide a subtree (change variable 
    * "hiden" to true)
    *
    * *************************************/

   public void setHidenSubtree(boolean e)
   {
	if (this!=null)
	{
		this.hiden = e;
		if (this.child!=null)
		{
			this.child.setHidenSubtree(e);

			if (this.child.getSibling() != null)
			{
				this.child.getSibling().setHidenSubtree(e);

				if (this.child.getSibling().getSibling()!=null)
					this.child.getSibling().getSibling().setHidenSubtree(e);
			}
		}			
	}
   }
  


   /***************************************
    *
    * set a subtree animated (change variable
    * "hiden" to true)
    *
    * ************************************
    * @param e
    */

   public void setAnimatedSubtree(boolean e)
   {
	if (this!=null)
	{
		this.animated = e;
		if (this.child!=null)
		{
			this.child.setHidenSubtree(e);

			if (this.child.getSibling() != null)
			{
				this.child.getSibling().setAnimatedSubtree(e);

				if (this.child.getSibling().getSibling()!=null)
					this.child.getSibling().getSibling().setAnimatedSubtree(e);
			}
		}
	}
   }



   public Object clone()
   {
	   DefaultGraphCell cell;
	   Boolean ev   = new Boolean(this.evaluated);
	   boolean eval = ev.booleanValue();

	   if (this.parent != null)
	   {
		   if (this.child!=null && this.sibling == null)
	   	   {
			if (vertice != null) { cell = (DefaultGraphCell)vertice.clone(); }
			else 		     { cell = new DefaultGraphCell();		}
			cell.addPort();
	   		return new CParentNode(this.label,(CParentNode)this.parent,
				   (CParentNode)this.child.clone(),null,hiden,(Point)this.position.clone(),
				   (Point)this.offset.clone(), this.width, this.height, this.level, this.contour,
				   cell,this.value,eval, this.animated);
	   	   }
		   else if (this.child==null && this.sibling != null)
		   {
			if (vertice != null) { cell = (DefaultGraphCell)vertice.clone(); }
			else 		     { cell = new DefaultGraphCell();		}
			cell.addPort();
		   	return new CParentNode(this.label,(CParentNode)this.parent,
				   null,(CParentNode)this.sibling.clone(),hiden,(Point)this.position.clone(),
				   (Point)this.offset.clone(), this.width, this.height, this.level, this.contour,
				   cell,this.value,((Boolean)eval).booleanValue(), this.animated);
		   }
		   else if (this.child==null && this.sibling == null)
		   {
			if (vertice != null) { cell = (DefaultGraphCell)vertice.clone(); }
			else 		     { cell = new DefaultGraphCell();		}
			cell.addPort();
			return new CParentNode(this.label,(CParentNode)this.parent,
						  null,null,hiden,(Point)this.position.clone(),
				   (Point)this.offset.clone(), this.width, this.height, this.level, this.contour,
				   cell,this.value,eval, this.animated);
		   }
		   else 
		   {
			if (vertice != null) { cell = (DefaultGraphCell)vertice.clone(); }
			else 		     { cell = new DefaultGraphCell();		}
			cell.addPort();
			return new CParentNode(this.label,(CParentNode)this.parent,
				   (CParentNode)this.child.clone(),(CParentNode)this.sibling.clone(),
				   hiden,(Point)this.position.clone(),
				   (Point)this.offset.clone(), this.width, this.height, this.level, this.contour,
				   cell,this.value,eval, this.animated);
		   }
	   }
	   else if (this.child!=null)
	   {
		if (vertice != null) { cell = (DefaultGraphCell)vertice.clone(); }
		else 		     { cell = new DefaultGraphCell();		}
		cell.addPort();

	   	return new CParentNode(this.label,null,
				   (CParentNode)this.child.clone(),null,hiden,(Point)this.position.clone(),
				   (Point)this.offset.clone(), this.width, this.height, this.level, this.contour,
				   cell,this.value,eval, this.animated);
	   }
	   else if (this.sibling!=null)
	   {		
		if (vertice != null) { cell = (DefaultGraphCell)vertice.clone(); }
		else 		     { cell = new DefaultGraphCell();		}
		cell.addPort();
		
	   	return new CParentNode(this.label,null,
				   null,(CParentNode)this.sibling.clone(),hiden,(Point)this.position.clone(),
				   (Point)this.offset.clone(), this.width, this.height, this.level, this.contour,
				   cell,this.value,eval, this.animated);
	   }
	   else
	   {
		   return null;
	   }
   }

   public void layout(CParentNode t) {
	CParentNode c;

	if(t == null) {
	    return;
	}

        c = t.child;
        while(c != null) {
 	    layout(c);
	    c = c.sibling;
	}

	if (t.child != null)
	{
	   attachParent(t, join(t));
	  // System.out.println("Join: " + t.label + " - " + join(t));
	} 
	else 
	{
	   layoutLeaf(t);
	}
    }

    
    private void attachParent(CParentNode t, int h) {
	int x, y1, y2;

	if (t.hiden) {
		x = t.border +  parent_dist;
		y2 = (h - t.height) / 2 - t.border;
		y1 = y2 + t.height + 2 * t.border - h;
		t.child.offset.x = 0;
		t.child.offset.y = 0;
	}
	else {
		x = t.border +  parent_dist;
		y2 = (h - t.height) / 2 - t.border;
		y1 = y2 + t.height + 2 * t.border - h;
		t.child.offset.x = x + t.width;
		t.child.offset.y = y1;
	}
	t.contour.upper_head = new CPolyLine(t.width, 0, new CPolyLine(x, y1, 
	  t.contour.upper_head));
	t.contour.lower_head = new CPolyLine(t.width, 0, new CPolyLine(x, y2, 
	  t.contour.lower_head));
    }

      private void layoutLeaf(CParentNode t) {
	t.contour.upper_tail = new CPolyLine(t.width + 2 * t.border, 0, null);
	t.contour.upper_head = t.contour.upper_tail;
	t.contour.lower_tail = new CPolyLine(0, -t.height - 2 * t.border, null);
	t.contour.lower_head = new CPolyLine(t.width + 2 * t.border, 0, t.contour.lower_tail);
    }


    private int join(CParentNode t) {
	CParentNode c;
	int d, h, sum;

	c = t.child;
	t.contour = c.contour;
	sum = h = c.height + 2 * c.border;
	c = c.sibling;
	while(c != null) {
	    d = merge(t.contour, c.contour);

	    if (t.hiden) {
		    c.offset.y = 0;
		    c.offset.x = 0;
		    h = 0;
		    sum += 0;
	    }
	    else {
		    c.offset.y = d + h;
		    c.offset.x = 0;
		    h = c.height + 2 * c.border;
		    sum += d + h;
	    }
		    c = c.sibling;
    	   }

	return sum;
    }

  
    private int merge(CPolygon c1, CPolygon c2) {
	int x, y, total, d;
	CPolyLine lower, upper, b;
	
	x = y = total = 0;
	upper = c1.lower_head;
	lower = c2.upper_head;
	
	while(lower != null && upper != null) {	// compute offset total 
	    
	    d = offset(x, y, lower.dx, lower.dy, upper.dx, upper.dy);
	    y += d;
	    total += d;

	    if(x + lower.dx <= upper.dx) {
		y += lower.dy;
		x += lower.dx;
		lower = lower.link;
	    } else {
		y -= upper.dy;
		x -= upper.dx;
		upper = upper.link;
	    }
	}

	// store result in c1 
	
	if(lower != null) {
	    b = bridge(c1.upper_tail, 0, 0, lower, x, y);
	    c1.upper_tail = (b.link != null) ? c2.upper_tail : b;
	    c1.lower_tail = c2.lower_tail;
	} else {	// (upper) 
	    b = bridge(c2.lower_tail, x, y, upper, 0, 0);
	    if(b.link == null) {
		c1.lower_tail = b;
	    }
	}

	c1.lower_head = c2.lower_head;
	
	return total;
    }

   
    private int offset(int p1, int p2, int a1, int a2, int b1, int b2) {
	int d, s, t;

	if(b1 <= p1 || p1 + a1 <= 0) {
	    return 0;
	}

	t = b1 * a2 - a1 * b2;
	if(t > 0) {
	    if(p1 < 0) {
		s = p1 * a2;
		d = s / a1 - p2;
	    } else if(p1 > 0) {
		s = p1 * b2;
		d = s / b1 - p2;
	    } else {
		d = -p2;
	    }
	} else if(b1 < p1 + a1) {
	    s = (b1 - p1) * a2;
	    d = b2 - (p2 + s / a1);
	} else if(b1 > p1 + a1) {
	    s = (a1 + p1) * b2;
	    d = s / b1 - (p2 + a2);
	} else {
	    d = b2 - (p2 + a2);
	}

	if(d > 0) {
	    return d;
	} else {
	    return 0;
	}
    }

     private CPolyLine bridge(CPolyLine line1, int x1, int y1, CPolyLine line2, int x2, int y2) {
	int dy, dx, s;
	CPolyLine r;

	dx = x2 + line2.dx - x1;
	if(line2.dx == 0) {
	    dy = line2.dy;
	} else {
	    s = dx * line2.dy;
	    dy = s / line2.dx;
	}

	r = new CPolyLine(dx, dy, line2.link);
	line1.link = new CPolyLine(0, y2 + line2.dy - dy - y1, r);

	return r;
    }

  public void plantTree(CParentNode t, int off_x, int off_y) {
	CParentNode c, s;
	int cur_y;

	t.position.x = off_x + t.offset.x;
	t.position.y = off_y + t.offset.y;

	/* Plant child node */
	c = t.child;
	if(c != null) {
	    plantTree(c, t.position.x, t.position.y);

	    /* Plant sibling nodes */
	    s = c.sibling;
	    cur_y = t.position.y + c.offset.y;
	    while(s != null) {
	        plantTree(s, t.position.x + c.offset.x, cur_y);
		cur_y += s.offset.y;
		s = s.sibling;
	    }
	}
    }



  @Override
  public String toString(){
    CTreeNode x = (CTreeNode) this.getLabel();
    return x.getSymbol().getName();
  }
}
