package Alma;

import java.util.ArrayList;
import java.util.Iterator;

import java.awt.Color;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;

import javax.swing.BorderFactory;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphLayoutCache;
import org.jgraph.graph.GraphModel;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.CellViewFactory;
import org.jgraph.graph.DefaultCellViewFactory;
import org.jgraph.graph.CellView;
import org.jgraph.graph.CellHandle;
import org.jgraph.graph.EdgeView;
import org.jgraph.graph.Edge;
import org.jgraph.graph.GraphContext;
import org.jgraph.graph.ParentMap;

import java.util.List;

class MyMouseListener extends MouseAdapter {
	private JGraph graph;
	private GraphModel model;

	public void setGraph(JGraph g)
	{
		this.graph = g;
	}
	
	public void setModel(GraphModel m)
	{
		this.model = m;
	}

	/**Recursive algorithm to select nearby cells
	 * @param mycell - the cell we are currently testing
	 * @param level - the decrementing level of depth we are at. when we get to 0, we are not going to look any deeper
	 * @param onlychildren - we only want to look at children, not parents
	 * @param unexpanded - view even the unexpanded nodes. If a cell is unexpanded, we will still select its children
	 * @param group - a vector that we are putting the selected cells into
	 */
	public void select(DefaultGraphCell mycell,int level, boolean onlychildren, boolean unexpanded, ArrayList group){
	   	int i;
		
		GraphModel model = this.graph.getModel();

		for (i= 0; i < model.getChildCount(mycell); i++)
		{
			//go through all the children (the ports)
		         Iterator edges = model.edges(model.getChild(mycell,i));//loop through all the ports' edges
		         while(edges.hasNext()) 
			   {
		            DefaultEdge edge = (DefaultEdge)edges.next();
		            DefaultGraphCell source = (DefaultGraphCell)model.getParent(model.getSource(edge));
		            DefaultGraphCell target = (DefaultGraphCell)model.getParent(model.getTarget(edge));
		            if (level>=0)
			    {
				   //if we haven't reached the bottom level, and either we the current cell is already expanded OR we don't care if it's expanded or not   
		               if (target!=mycell)
			       {
		                  group.add(edge);//add the edge to the list
		                  select(target,level-1,onlychildren, unexpanded, group);//select the next level below
		               }
		               if(!onlychildren && source!=mycell)
			       {
		                  select(source,level-1,onlychildren,unexpanded,group);//select the next level above
		                  group.add(edge);//add the edge to the list
		               }
		            }
		         
		      }
		}
   		group.add(mycell);//add the node to the list
	}	


/**
	 * Overrides the parent example group method to move all edges that cross
	 * the group (eg. from outside to inside and vice versa) to the group's
	 * port.
	 */
	public void group(Object[] children) {
		// Order Cells by Model Layering
		children = graph.order(children);
		// If Any Cells in View
		if (children != null && children.length > 0) {
			double gs2 = 2 * graph.getGridSize();
			Rectangle2D collapsedBounds = graph.getCellBounds(children);
			collapsedBounds.setFrame(collapsedBounds.getX(), collapsedBounds
					.getY(), Math.max(collapsedBounds.getWidth() / 4, gs2),
					Math.max(collapsedBounds.getHeight() / 2, gs2));
			graph.snap(collapsedBounds);
			DefaultGraphCell group = createGroupCell(collapsedBounds);
			if (group != null && children != null && children.length > 0) {
				// Create the group structure
				ParentMap pm = new ParentMap();
				for (int i = 0; i < children.length; i++) {
					pm.addEntry(children[i], group);
				}
				graph.getGraphLayoutCache().insert(new Object[] { group },
						null, null, pm);
			}
		}
	}

	/**
	 * Hook from GraphEd to create a new group cell
	 */
	protected DefaultGraphCell createGroupCell(Rectangle2D collapsedBounds) {
		DefaultGraphCell group = new DefaultGraphCell();
		group.addPort();
		GraphConstants.setInset(group.getAttributes(), 10);
		GraphConstants.setBackground(group.getAttributes(), new Color(240, 240,
				255));
		GraphConstants.setBorderColor(group.getAttributes(), Color.black);
		GraphConstants.setOpaque(group.getAttributes(), true);
		GraphConstants.setBorder(group.getAttributes(), JGraphShadowBorder
				.getSharedInstance());
		GraphConstants.setBounds(group.getAttributes(), collapsedBounds);
		return group;
	}


	// Determines if a Cell is a Group
	public boolean isGroup(Object cell) {
		// Map the Cell to its View
		CellView view = graph.getGraphLayoutCache().getMapping(cell, false);
		if (view != null)
			return !view.isLeaf();
		return false;
	}
	
	public void collapseCell(DefaultGraphCell cell)
	{
		ArrayList group = new ArrayList();
		select(cell,20,true,false,group);
//		System.out.println("Count: " + group.size());
		this.graph.getGraphLayoutCache().setVisible(group.toArray(),false);
		this.graph.setSelectionCells(group.toArray());
		group(graph.getSelectionCells());
//		graph.getGraphLayoutCache().collapse(graph.getSelectionCells());
		//this.graph.getGraphLayoutCache().setVisible(cell,false);
         }   
   
	public void expandCell(DefaultGraphCell cell)
	{
		ArrayList group = new ArrayList();
		this.graph.getGraphLayoutCache().setVisible(cell,true);
	        select(cell,20,true,false,group);
	        this.graph.getGraphLayoutCache().setVisible(group.toArray(),true);   
   	} 

	public void mousePressed(MouseEvent e) {
		Object cell = null;
		Object[] cells;
		int i;

	//	if (!SwingUtilities.isRightMouseButton(e)) {	
	//	if (e.getClickCount()==2)
	//	{
			if (graph != null)
			{	
				cell = graph.getFirstCellForLocation(e.getX(),e.getY()); }
				if (cell != null) { if (!isGroup(cell)) 	{ collapseCell((DefaultGraphCell)cell); }
			}
	//	}
	//	}
	}
}

public class Graph {

	private JGraph graph;
	private Object[] cells;
	private GraphModel model;
		
	public Graph() {
		this.graph = createGraph();
	}

	public JGraph getGraph()
	{
		return this.graph;
	}
	
	public JGraph createGraph()
	{
		// Construct Model and Graph
		model = new DefaultGraphModel();
		// the cell view factory returns the default views for vertices, edges
		// and ports
		CellViewFactory cvf = new MyCellViewFactory();
		// thats the view of the graph
		GraphLayoutCache glc = new GraphLayoutCache(model, cvf);
		
		JGraph graph = new JGraph(model,glc);

		// Control-drag should clone selection
		graph.setCloneable(true);

		// Enable edit without final RETURN keystroke
		graph.setInvokesStopCellEditing(false);

		// When over a cell, jump to its default port
		graph.setJumpToDefaultPort(true);

		// set editable to false
		graph.setEditable(false);
		//graph.setMoveable(false);
		graph.getGraphLayoutCache().setSelectsAllInsertedCells(false);
		graph.setAntiAliased(true);
		MyMouseListener myMouse = new MyMouseListener();
		myMouse.setGraph(graph);
		myMouse.setModel(model);
		
		graph.addMouseListener(myMouse);

		return graph;
	}

	// change a color --- means that node will be calculated
	public void changeColor(DefaultGraphCell cell, Color c)
	{
		// change color after update node
		GraphConstants.setGradientColor(cell.getAttributes(),c);
	}

	// change foreground color
	public void setForeground(DefaultGraphCell cell, Color c)
	{
		// change color after update node
		GraphConstants.setForeground(cell.getAttributes(),c);
	}
	

	// change the position of cell
	public void changeBounds(DefaultGraphCell cell, int x, int y, int width, int height)
	{
		GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(x,y,width,height));
		
	}


	public void setGraph(ArrayList vertex)
	{
		int i=0, size1; 
		boolean flag = false;

		if (this.cells!=null) {	
			this.graph.getGraphLayoutCache().remove(this.cells);			
			model = new DefaultGraphModel();
			flag = true;	}

		size1 = vertex.size();
		this.cells = new Object[size1];
		
		for ( i = 0; i < size1; i++ )
		{
			this.cells[i] = (DefaultGraphCell)vertex.get(i);
		}

		this.graph.getGraphLayoutCache().insert(this.cells);
	
	}
	
	public static DefaultGraphCell createRectangle(String name, double x,
			double y, double w, double h, Color bg, boolean raised) {

		// Create vertex with the given name
		DefaultGraphCell cell = new DefaultGraphCell(name);

		// Set bounds
		GraphConstants.setBounds(cell.getAttributes(), new Rectangle2D.Double(
				x, y, w, h));

		// Set fill color
		if (bg != null) {
			GraphConstants.setGradientColor(cell.getAttributes(), bg);
			GraphConstants.setOpaque(cell.getAttributes(), true);
		}

		// Set raised border
		if (raised)
		{
			GraphConstants.setBorder(cell.getAttributes(), BorderFactory
					.createRaisedBevelBorder());
		}
		else
		{
			// Set black border
			GraphConstants.setBorderColor(cell.getAttributes(), Color.black);
		}

		// Add a Floating Port
		cell.addPort();

		return cell;
	}

	public static DefaultGraphCell createEllipse(String name, double x,
			double y, double w, double h, Color bg, boolean raised) {

		// Create vertex with the given name
		DefaultGraphCell cell = new MyEllipseGraphCell(name,new Rectangle2D.Double(
				x, y, w, h));

		// Set fill color
		if (bg != null) {
			GraphConstants.setGradientColor(cell.getAttributes(), bg);
			GraphConstants.setOpaque(cell.getAttributes(), true);
		}

		// Set raised border
		if (raised)
		{
			GraphConstants.setBorder(cell.getAttributes(), BorderFactory
					.createRaisedBevelBorder());
		}
		else
		{
			// Set black border
			GraphConstants.setBorderColor(cell.getAttributes(), Color.black);
		}

		// Add a Floating Port
		cell.addPort();

		return cell;
	}

	// Create a Group that Contains the Cells
	public void group(Object[] cells) {
		// Order Cells by Model Layering
		cells = graph.order(cells);
		// If Any Cells in View
		if (cells != null && cells.length > 0) {
			DefaultGraphCell group = new DefaultGraphCell();
			// Insert into model
			graph.getGraphLayoutCache().insertGroup(group, cells);
		}
	}

// Hook for subclassers
	protected JGraph createGraph2() {
		JGraph graph = new MyGraph(new MyModel());
		graph.getGraphLayoutCache().setFactory(new DefaultCellViewFactory() {

			// Override Superclass Method to Return Custom EdgeView
			protected EdgeView createEdgeView(Object cell) {

				// Return Custom EdgeView
				return new EdgeView(cell) {

					/**
					 * Returns a cell handle for the view.
					 */
					public CellHandle getHandle(GraphContext context) {
						return new MyEdgeHandle(this, context);
					}

				};
			}
		});
		return graph;
	}
	
}

// Defines a Graph that uses the Shift-Button (Instead of the Right
	// Mouse Button, which is Default) to add/remove point to/from an edge.
	class MyGraph extends JGraph {

		// Construct the Graph using the Model as its Data Source
		public MyGraph(GraphModel model) {
			this(model, null);
		}

		// Construct the Graph using the Model as its Data Source
		public MyGraph(GraphModel model, GraphLayoutCache cache) {
			super(model, cache);
			// Make Ports Visible by Default
			setPortsVisible(true);
			// Use the Grid (but don't make it Visible)
			setGridEnabled(true);
			// Set the Grid Size to 10 Pixel
			setGridSize(6);
			// Set the Tolerance to 2 Pixel
			setTolerance(2);
			// Accept edits if click on background
			setInvokesStopCellEditing(true);
			// Allows control-drag
			setCloneable(true);
			// Jump to default port on connect
			setJumpToDefaultPort(true);
		}

	}

	//
	// Custom Edge Handle
	//

	// Defines a EdgeHandle that uses the Shift-Button (Instead of the Right
	// Mouse Button, which is Default) to add/remove point to/from an edge.
	class MyEdgeHandle extends EdgeView.EdgeHandle {

		/**
		 * @param edge
		 * @param ctx
		 */
		public MyEdgeHandle(EdgeView edge, GraphContext ctx) {
			super(edge, ctx);
		}

		// Override Superclass Method
		public boolean isAddPointEvent(MouseEvent event) {
			// Points are Added using Shift-Click
			return event.isShiftDown();
		}

		// Override Superclass Method
		public boolean isRemovePointEvent(MouseEvent event) {
			// Points are Removed using Shift-Click
			return event.isShiftDown();
		}

	}

	//
	// Custom Model
	//

	// A Custom Model that does not allow Self-References
	class MyModel extends DefaultGraphModel {
		// Override Superclass Method
		public boolean acceptsSource(Object edge, Object port) {
			// Source only Valid if not Equal Target
			return (((Edge) edge).getTarget() != port);
		}

		// Override Superclass Method
		public boolean acceptsTarget(Object edge, Object port) {
			// Target only Valid if not Equal Source
			return (((Edge) edge).getSource() != port);
		}
	}

