package Alma;

import java.util.ArrayList;

import Lisa.Parser.CTreeNode;

import javax.swing.JTextPane;
import javax.swing.text.Element;
import javax.swing.text.Highlighter;
import javax.swing.text.BadLocationException;

import java.awt.Color;

public class Motor
{
	private Visualize vis;
	private Object value;

	public Motor()
	{
		this.vis      = new Visualize(new Graph());
		this.value    = null;
	}

	/** creates the graph, saving cells and connections in
	 * an ArrayList
	 *
	 * @param vertices --- where the info is saved
	 * 
	 * */
	public void makeGraph(CParentNode t, ArrayList vertices)
   	{
		CParentNode n1;
		
		vertices.clear();
		vis.initLevel();
		vis.setParent(t);
		n1 = (CParentNode)t.clone();
		vis.createCells(n1,vertices);
		
		copyLevels(n1,t);
		
		vis.makeConnect(t,vertices);
   	}

	/** copies info about the level of a node in the
	 * tree. (it's used to collect the cells
	 * and then make connections between them in the graph
	 *
	 * @param from --- levels to copy from
	 *        to   --- levels to copy to
	 * */
	public void copyLevels(CParentNode from, CParentNode to)
   	{
		if (from!=null)
	   	{
			to.setLevel(from.getLevel());
		     	copyLevels(from.getChild(),to.getChild());
			copyLevels(from.getSibling(),to.getSibling());
		}
   	}

	/** finds the mapping between the arguments defined in function
	 * and the expressions passed in the function as arguments
	 *
	 * @param node - node to process and find arguments
	 *
	 * @return void
	 *
	 * continues processing until we find a node diferent from stats
	 * */
	public void processNodes(CTreeNode node, ArrayList result)
	{
		int i;

		if (node!= null)
		{
			if (node.getSymbol().getName().equals("STATS")) {
				for ( i = 0; i < node.getNodes().length; i++)
				{
					processNodes(node.getNodeAt(i),result);
				}
			}
			else 	{ result.add(node); }
		}
	}

	/** changed the nodes acording it's position
	 *
	 * when we declare a function, the arguments are a list of statements
	 * when we call a function, the arguments are a list of expressions
	 *
	 * @param node - node to substitute vars with expressions
	 * @param args - list of arguments
	 * @param expr - list of arguments
	 *
	 * each variable in node is changed with an assignment
	 * */

	public CTreeNode mappingArgsExpr(CTreeNode node, ArrayList expr, int index)
	{
		int i;
		String name;

		// is not null
		if (node!= null)
		{
			name = node.getSymbol().getName();
			// have arguments (this is, when we do not have arguments
			// we use a CTreeNode with symbol "null"
			if (!name.equals("null")) {
				if (node.getSymbol().getName().equals("STATS")) {
					for ( i = 0; i < node.getNodes().length; i++)	{
						CTreeNode n = mappingArgsExpr(node.getNodeAt(i), expr, index);
						node.setNodeAt(n,i);
						index++;
					}
				}
				else 	{ 
					// in case of functions called recursively
					// System.out.println("AQUI mapping " + name);
					if (node.getSymbol().getName().equals("ASSIGN"))	{
						CTreeNode n = node.getNodeAt(0);
						CAlmaNode newNode = new CAssignNode((CAlmaNode)n, (CAlmaNode)expr.get(index));
						node = newNode;
					}
					else if (name.equals("VAR")) {
						CAlmaNode newNode = new CAssignNode((CAlmaNode)node,
								(CAlmaNode)expr.get(index));
						node = newNode;
					}
				}
			}
		}
		return node;
	}

	/** find a specific node in a tree to return the value at that node
	 *
	 * @param tree   - the tree where we are searching
	 *        node   - node to find
	 *
	 * @return value of node
	 * */
	public void findNode(CParentNode tree, CTreeNode node)
	{
		if (tree!=null)
		{
			if (((CTreeNode)tree.getLabel()).toString().equals(node.toString())) {
				value  = this.getCorrectValue(tree);
			}
			else {
				findNode(tree.getChild(),node); 
				findNode(tree.getSibling(),node); 
			}
		}
	}

	/** gets the current value of a node **/
	public Object getValue()
	{
		return this.value; 
	}

	/** sets the current value to null **/
	public void setValue()
	{
		this.value = null; 
	}


	/** find a specific node in a tree to return the value at that node
	 *
	 * @param tree   - the tree where we are searching
	 *        node   - node to find
	 * */
	public void setValue(CParentNode tree, CTreeNode node, Object value)
	{
		if (tree!=null)
		{
			if (((CTreeNode)tree.getLabel()).toString().equals(node.toString()))
			{
				tree.setValue(value);
				//System.out.println("Tree: " + tree.getValue());
			}
			else
			{
				setValue(tree.getChild(),node,value);
				setValue(tree.getSibling(),node,value);
			}
		}
	}

	/** finds a value 
	 *
	 *
	 * in case of find a tree in the following form
	 *
	 * 	b				or		
	 *    /
	 *   /
	 * []							return --- statements --- constant 0
	 *   \
	 *    \
	 *     Statements --- Constant 0
	 *
	 *     we traverse the tree until we get the node "Constant 0"
	 *
	 *     **/
	public Object getCorrectValue(CParentNode tree)
	{
		CParentNode auxT = tree;
		boolean find     = false;
		Object value     = null;

		while (auxT!=null && !find) {
			if (auxT.getValue() != null) {
				find = true;
				value = auxT.getValue();
			}
			else { auxT = auxT.getChild(); }
		}
		return value;
	}

	public void highlightPanel(JTextPane t, int line) {
		
		Element rootDoc = t.getDocument().getDefaultRootElement();
		Element lineElement = rootDoc.getElement( line - 1 );
		Highlighter hl = t.getHighlighter();
	        int start = lineElement.getStartOffset();
	        int end   = lineElement.getEndOffset();


		try {
			hl.removeAllHighlights();
			//System.out.println("AQUI: " + line + " , " + start);
			hl.addHighlight(start,end,new LineHighlighter(t,Color.RED));
			// Remove any existing highlights for last word
			Highlighter.Highlight[] highlights = hl.getHighlights();
			if (highlights.length > 1) {
		        for (int i = 0; i < 1; i++) {
		  	      Highlighter.Highlight h = highlights[i];
				//      if (h.getPainter() instanceof UnderlineHighlighter.UnderlineHighlightPainter) {
			        hl.removeHighlight(h);

      			}
			}
    }

		 catch (BadLocationException ble) {
		    System.err.println("Couldn't insert initial text.");
		}
	}	
}

