package Alma;
// Pair.java

/*
   This file shows how you can define a new class, and
   gives you some simple examples of java "methods".

   You should compile it as normal, using the command
   "javac Pair.java", but since it doesn't contain a
   "public static void main ( String args[] )" method,
   you cannot run it directly with "java Pair".  
   (Try it and see what error message you get!)
*/

// This is a class to represent a pair of Objects.
// See TestPair1.java for an example of its useage.

public class Pair
{
    // ===================================================================

    // An Object of class Pair "contains" two separate
    // Objects.  These are going to be stored as special
    // local variables.  As always, we must declare them 
    // before we use them!

    // The only new feature here is the word "private"
    // which means that only code within the in this class
    // file can access this data directly.  That means the
    // data is protected and no other program can mess up
    // our data accidently (or on purpose).  The keyword
    // private is not required, but STRONGLY recommended.

    private Object ob1;
    private Object ob2;

    // The scope of ob1, ob2 is the code between the { ... }
    // immediately after the "class Pair" line above.

    // ===================================================================

    // The next thing we need to do is to say how to make
    // a new Pair object.  In other words, we must tell the
    // computer what it should do when it sees a line like
    // "new Pair(x,y)".  This section of the class Pair is
    // called a "constructor", since it says how to construct 
    // a Pair.  In our case here, all we want to do is to store
    // the data x,y in our private variables ob1, ob2.

    public Pair(Object x, Object y)
    {
        ob1 = x;
        ob2 = y;
    }

    // There are a couple of things to note about the constructor.

    // First, you have to tell it what sort of data to accept
    // as its inputs.  Here, both inputs are plain Objects,
    // so we write "Object x" and "Object y".  This also tells
    // the compiler to recognise the variables x and y.  (In this
    // case, the scope of the variables x,y is the {...} block
    // immediately following the "public Pair(Object x, Object y)" line.)

    // The other thing to notice is the word "public".  This allows
    // (or even encourages) an external program to use this constructor.
    // Contrast this with "private" which would disallow such use.
    // You are STRONGLY advised to declare all data and methods in 
    // every class you write as "public" or "private".  

    // Here is another constructor: it tells java how to make
    // a new Pair object given an old Pair object.  In this case
    // the new pair is a copy of the old one.

    public Pair(Pair p)
    {
        ob1 = p.first();  // the "first" method extracts the first
     	                  // member of the pair, and is defined below
        ob2 = p.second(); //"second" is also defined below   
    }

    // ===================================================================

    // The rest of this class defines the methods that a Pair object
    // will have.  These methods can be public (anyone can use them)
    // or private (only code in this class can use them).  They
    // may or may not return some data, and if they do you have
    // to say what type of data is returned.  Also, they may or may not
    // use some input data, and again yoy have to say what type of
    // data should be accepted.

    // For example, here is a method called "first" that is public,
    // returns an Object, and does not require any input data.
    // It simply returns the value of ob1.

    public Object first()
    {
        return ob1;
    }

    // Similarly, a method called "second":

    public Object second()
    {
        return ob2;
    }

    // This method does not return any data.  (That's
    // what the special word void means.)  But it does
    // require input data, an Object called ob.
    // it simply sets ob1 to the input data.

    public void setFirst(Object ob)
    {
        ob1 = ob;
    }

    // Similarly, this method sets ob2 to the input data.

    public void setSecond(Object ob)
    {
        ob2 = ob;
    }

    // Finally, the toString method returns a string representation
    // of the Pair object.  This is useful if we want to print
    // our Pair object out.  The method is public, returns a String,
    // and doesn't require any input data.  The code defining this
    // method uses the convenient fact that all Objects already
    // hav a "toString" method which we can use.

    public String toString()
    {
        String s ;    // declare a String variable
	s = "(" + ob1.toString() + "," + ob2.toString() + ")"; 
                      // note the use of + to concatenate (join)
	              // separate strings here
        return s;     // return our String object
    }

} 
