/*
 * This class is used to traverse the DAST and perform <br/>
 * the animation of the problem domain.<br/>
 * <br/>
 * For each element in the tree, it is needed to see if <br/>
 * is to be animated or if its time of animation is not arrived yet.<br/>
 * 
 */

package Alma.animation;

import Alma.CAlmaNode;
import Alma.CParentNode;
import Alma.Motor;
import Alma.Rewrite;
import Alma.TAB;
import Alma.animation.patterns.APAggregator;
import Alma.animation.patterns.AnimationPattern;
import Lisa.Parser.CTreeNode;
import java.util.ArrayList;

/**
 *
 * @author Nuno Oliveira
 */
public class Animate {

    private TAB idT; //used to store the id Table...
    private DrawingPanel dp; //used to store the panel to be repainted
    private Rewrite rew; //used to store the rewrite that comes from the backend
                         //and which will be used to set the actors to paint in the
                         //drawing panel
    private Motor motor; //used to store the object that is used to evaluate nodes' value
    
    public Animate(TAB ti, DrawingPanel dp, Rewrite rew) {
    
        this.idT = ti;
        this.dp = dp;
        this.rew = rew;
        this.motor = new Motor();
    }

    public DrawingPanel getDp() {
        return dp;
    }

    public void setDp(DrawingPanel dp) {
        this.dp = dp;
    }

    public TAB getIdT() {
        return idT;
    }

    public void setIdT(TAB idT) {
        this.idT = idT;
    }



    /**
     * Set FUNC nodes already animated.
     * Which implies that the subtree is also animated
     */
    private void avoidNodes(CParentNode node){
        CTreeNode ctn = (CTreeNode)node.getLabel();
        String name = ctn.getSymbol().getName();
        if(name.equals("FUNC") ){
            node.setAnimated(true);
            performAnimation(node.getParent());
        }
    }



    /**
     *
     * This method is used to evaluate a IF node.
     * Here is controled the flux of the program.
     * The first child is assumed to be already animated
     * The simple algorithtm is: if the expression
     * evaluates to true, then we set the 3rd child as
     * animated and go evaluate the 2nd one. Otherwise,
     * we set the 2nd child as animated, and go evaluate the
     * 3rd child.
     *
     * @param node the node to evaluate
     */
    private void ifNode(CParentNode node){
        CTreeNode ctn = (CTreeNode)node.getLabel();
        String name = ctn.getSymbol().getName();

        if(name.equals("IF")){
            //System.out.println("Entered IF...");
            Object value = motor.getCorrectValue(node.getChild());
            //If the IF expression is evaluated as true
            if(value.toString().equals("true")){
                //System.out.println("Evaluated as True...");
                if(node.haveSecondSibling()) {
                    //set the second sibling (ELSE) as animated
                    //System.out.println("filho2 -> set animated");
                    node.getChild().getSibling().getSibling().setAnimated(true);
                }
                //The first sibling is then animated...
                if(!node.getChild().getSibling().isAnimated()){
                    //System.out.println("filho1 ! animated -> perform...");
                    performAnimation(node.getChild().getSibling());
                }
                else{//in the end, animate the IF node
                    //System.out.println("Do animation of IF");
                    doAnimation(node);
                    //System.out.println("Do animation of IF parent");
                    performAnimation(node.getParent());
                }
            }
            else{
                //System.out.println("Evaluated as False");
                if(node.haveSecondSibling()){
                    //set the first sibling as animated
                    //System.out.println("filho1 -> set animated");
                    node.getChild().getSibling().setAnimated(true);
                }
                //The second sibling is then animated...
                if(!node.getChild().getSibling().getSibling().isAnimated()){
                    //System.out.println("filho2 ! animated -> perform...");
                    performAnimation(node.getChild().getSibling().getSibling());
                }
                else{
                    //in the end, animate the IF node
                    //System.out.println("Do animation of IF");
                    doAnimation(node);
                    //System.out.println("Do animation of IF parent");
                    performAnimation(node.getParent());
                }
            }
        }
        //System.out.println("Exited IF...");
    }



    /**
     * This method is used to evaluate a node of type WHILE
     * It assumes that the first child of the WHILE node is already
     * evaluated.
     * The algorithm is described shortly: If the conditional expression
     * evaluates to true, then we unanimate the 2nd child and its subtree
     * and process it; otherwise everything is animated, so we evaluate the
     * WHILE node and then it is set to animated...
     *
     * @param node the node to be evaluated
     */
    private void whileNode(CParentNode node) {
        CTreeNode ctn = (CTreeNode)node.getLabel();
        String name = ctn.getSymbol().getName();

        //If it is While node
        if(name.equals("WHILE")){
            //System.out.println("Entered While...");
            Object value = motor.getCorrectValue(node.getChild());
            //If the expression was evaluated to true
            if(value.toString().equals("true")){
                //System.out.println("Expression evaluated TRUE");
                //if have second child
                if(node.haveChildSibling()){
                    //only when the second child is animated, we set the while subtree to not animated
                    if(node.getChild().getSibling().isAnimated()){
                        //System.out.println("Start unanimating the subtree...");
                        //set the subtree unanimated (to be possible a new iteration)
                        unAnimateSubTree(node);
                        //and animate the second child
                        performAnimation(node.getChild().getSibling());
                    }
                    else {
                        //System.out.println("Animate the second child");
                        //animate the second child
                        performAnimation(node.getChild().getSibling());
                    }
                }
            }
            else{
                //If the expression evaluated to false
                //The expression is already animated
                //The second child must be set to animated also
                //System.out.println("Expression evaluated False");
                //System.out.println("filho 2 set as animated");
                node.getChild().getSibling().setAnimated(true);
                //animate the while node
                //System.out.println("animate the while node");
                doAnimation(node);
            }
        }
        //System.out.println("Exited While...");
    }



    /**
     * This method evaluates a normal node that does not has
     * any flux variation (different from While and If).
     * The tree is evaluated normaly with a traversal over the nodes.
     *
     * @param node the node to evaluate
     */
    private void normalNode(CParentNode node){
        CTreeNode ctn = (CTreeNode)node.getLabel();
        String name = ctn.getSymbol().getName();

        //If the node is not while or If
        if(!(name.equals("IF") && name.equals("WHILE"))){
            //if the node has a second child
            if(node.haveChildSibling()){//if5
                //System.out.println("entered: if5 -- filho2");
                //if the 2nd child is animated
                if(node.getChild().getSibling().isAnimated()){//if6
                    //if the node have a third child
                    if(node.haveSecondSibling()){ //if7
                        //System.out.println("entered: if7 -- filho3 ");
                        //if the the third child is animated
                        if(node.getChild().getSibling().getSibling().isAnimated()){ //if8
                            //System.out.println("entered: if8 -- filho3 animado " + node.getChild().getSibling().toString());
                            // we animate the node itself, because there is no more branches
                            // in the subtree of the node.
                            doAnimation(node);
                            // then we go over that node again and perform the animation
                            // for the case of being a STATS node, and then be possible the evaluation of the
                            // parent node.
                            performAnimation(node);
                        } //end if8
                        else{ //else8
                            //System.out.println("entered: else8 -- !filho3 animado " + node.getChild().getSibling().toString());
                            // If the third child is not animated
                            // we animate that child (and its subtree)
                            performAnimation(node.getChild().getSibling().getSibling());
                        } //end else8
                    } //end if7
                    else{ //else7
                        //System.out.println("entered: else7 -- !filho3 animado " + node.getChild().getSibling().toString());
                        // If the node has not 3rd child
                        // since the other children are already animated, then we animate the
                        // node itself
                        doAnimation(node);
                        // and then we perform a new evaluation of that node, to see
                        // if it is a STATS node...
                        performAnimation(node);
                    } //end else7
                }//end if6
                else{//else6
                    //System.out.println("entered: else6 -- !filho2 animado " + node.getChild().getSibling().toString());
                    // If the 2nd child is not animated,
                    // we evaluate it
                    performAnimation(node.getChild().getSibling());
                }//end else6
            }//end if5
            else{//else5
                //System.out.println("entered: else5 -- !filho2");
                // if there is not a second child
                // then we animate the node itself
                doAnimation(node);
                // and then perform an evaluation of it to determine if it is a
                // STATS node...
                performAnimation(node);
            }//end else5
        }
        
    }


    /**
     * This method is used to traverse the tree and to animate<br/>
     * the nodes whenever it is possible.<br/>
     * The algorithm is breath-first based and depends on the<br/>
     * state (animated or not animated) of the nodes.<br/>
     *<br/>
     * This method is used to evalueate a piece of the tree, at<br/>
     * each step of the animation.<br/>
     *
     * @param node the node to be evaluated
     */
    public void performAnimation(CParentNode node){
        //System.out.println("start performing -- " + node.toString());

        //If the node is not null
        if(node != null){//ife
//            try{
//                CAlmaNode almaNode = (CAlmaNode)((CTreeNode) node.getLabel());//.getNodeAt(0);
//                System.out.println(almaNode.toString() + " = " + node.toString());
//            }
//            catch(Exception e){
//                System.out.println(node.toString());
//            }

            //get the name of the node
            CTreeNode ctn = (CTreeNode)node.getLabel();
            String name = ctn.getSymbol().getName();
            //System.out.println(name);
            //avoid the evaluation of some nodes (for example: FUNC)
            avoidNodes(node);

            // if the node is CALL we animate it
            if(name.equals("CALL")){//ifCALL
                doAnimation(node);
            }//endifCALL
            else{ //elseCALL
                // If the node is not CALL
                // If the node is not animated
                if(!node.isAnimated()){//if2
                    //System.out.println("entered: if2 -- !animado");
                    // If the node has 1st child
                    if(node.haveChild()){//if3
                        //System.out.println("entered: if3 -- tem filho");
                        // If the first node is not animated
                        if(node.getChild().isAnimated()){//if4
                            //System.out.println("entered: if4 -- filho animado " + node.getChild().toString() );
                            // If the node is a IF node
                            if(name.equals("IF")){
                                // We execute the function to evaluate the node IF
                                ifNode(node);
                            }
                            else{
                                // If the node is WHILE
                                if(name.equals("WHILE")) {
                                    // We execute the function to evaluate the node WHILE
                                    whileNode(node);
                                }
                                 else {
                                    //Otherwise, we execute the normal traversal to the tree
                                     normalNode(node);
                                }
                            }
                        }//end if4
                        else{//else4
                            //System.out.println("entered: else4 -- !filho animado " + node.getChild().toString() );
                            // If the child is not animated
                            // We evaluate that 1st child node
                            performAnimation(node.getChild());
                        }//end else4
                    }//end if3
                    else{//else3
                        //System.out.println("entered: else3 -- !tem filho");
                        // If the node has not 1st child
                        // we aimate that node
                        doAnimation(node);
                        // and get back to the parent to evaluate it.
                        performAnimation(node.getParent());
                    }//end else3
                }//end if2
                else{//else2
                    // If the node is animated
                    // and it is a STATS node and this node has parent
                    if(name.equals("STATS") && node.getParent()!=null){
                        // We evaluate the parent
                        performAnimation(node.getParent());
                        // otherwise we would be stuck in a statement
                    }
                }//end else2
            }//end elseCALL
        }//end ife

        //System.out.println("end performing");

    }





    

    /**
     * This method is used to effectively animate the node<br/>
     * By processing the animation patterns that are associated <br/>
     * with the nodes. <br/>
     * <br/>
     * 1 - we get the patterns in the node <br/>
     * 2 - we animate each pattern <br/>
     * 3 - we collect the distinct actors that performs in the pattern <br/>
     * 4 - we set the actors in the drawing panel <br/>
     * 5 - we repaint the drawing panel <br/>
     * 6 - we also set the actors in the rewrite object so we can pass<br/>
     *     the most recent actors to a function CALL whenever the node<br/>
     *     occurs in the tree.<br/>
     *
     * @param node to be animated
     */
    private void doAnimation(CParentNode node) {
        //System.out.println("started doAnimation");
        try {
            CAlmaNode almaNode = (CAlmaNode)((CTreeNode) node.getLabel());//.getNodeAt(0);
            //almaNode.print();
            ArrayList<AnimationPattern> aps = almaNode.getAps();
            ArrayList<Actor> actors = new ArrayList<Actor>();


            //If there were used Animated AlmaNodes (with a list of patterns)
            if (aps.size()>0) {//if1
                //System.out.println("started rendering");
                for (AnimationPattern ap : aps) {//for
                    //System.out.println("AP_processing");

                    //Set the idTable (to access the values)
                    ap.setTab(idT);
                    if(!ap.isIsAggregator()){
                        ap.animate();
                        //System.out.println("AP_processED");
                        // If an actor is not yet in the List
                        // Notice that the Actors are compared by their position in the memory
                        // this happens because we need to see if it is really a different object
                        // we are trying to add to this list of Actors
                        if(ap.getObj()!=null){
                            if (!actors.contains(ap.getObj())) {//if2
                                //System.out.println("ola" + ap.getObj().toString());
                                actors.add(ap.getObj());
                            }//end if2
                        }
                    }
                    else{
                        //System.out.println("Animate.java(line415): Entrei Aqui");
                        APAggregator apagg = (APAggregator)ap;
                        //System.out.println(apagg.getTab().getEnv().toString());
                        //System.out.println(idT.getEnv().toString());
                        apagg.animate();
                        //System.out.println("Animate.java(line417): SIZE="+apagg.getAggregator().size());
                        for(Actor ac : apagg.getAggregator()){
                            actors.add(ac);
                        }
                    }
                    //System.out.println(actors);
                }//end for

                // Set the actors in the drawing panel for visual animation
                dp.setActors(actors);
                // Refresh the panel
                dp.makeAnimation();
                //Set the actors in the rewrite class to pass it to funcDialog and so on...
                rew.setActors((ArrayList<Actor>)actors.clone());
                //System.out.println("end rendereing");
            }//end if1
//            else {
//                System.out.println("APS: NULL");
//            }

            //Set the node animated (whether the node has animation patterns or not)
            node.setAnimated(true);

        } catch (Exception e) {
            // In case of some problem occur, we set the node to animated
            node.setAnimated(true);
        }
       // System.out.println("end doAnimation");
    }

    /**
     * This method sets the subtree of a node as not animated<br/>
     * That is, each node of the tree is set to not animated
     *
     * @param node the node to set the subtree as not animated
     */
    public void unAnimateSubTree(CParentNode node) {
        if(node!=null){
            node.setAnimated(false);
            if (node.haveChild()) {
                //System.out.println("startFILHO");
                //System.out.print("--");
                unAnimateSubTree(node.getChild());
                //System.out.println("endFILHO");
            }
            if (node.haveChildSibling()) {
                //System.out.println("startFILHO2");
                //System.out.print("----");
                unAnimateSubTree(node.getChild().getSibling());
                //System.out.println("endFILHO2");
            }
            if (node.haveSecondSibling()) {
                //System.out.println("startFILHO3");
                //System.out.print("------");
                unAnimateSubTree(node.getChild().getSibling().getSibling());
                //System.out.println("endFILHO3");
            }
        }
    }



    /**
     * This method prints the name of the nodes of a subtree of a node...
     *
     * @param t the node (tree) to print...
     */
    public void printSubTree(CParentNode t){
        if(t!=null){

            try {
                CAlmaNode node = (CAlmaNode) ((CTreeNode) t.getLabel()); //.getNodeAt(0);
                System.out.println(node);
                if (t.haveChild()) {
                    //System.out.println("startFILHO");
                    System.out.print("--");
                    printSubTree(t.getChild());
                    //System.out.println("endFILHO");
                }
                if (t.haveChildSibling()) {
                    //System.out.println("startFILHO2");
                    System.out.print("----");
                    printSubTree(t.getChild().getSibling());
                    //System.out.println("endFILHO2");
                }
                if (t.haveSecondSibling()) {
                    //System.out.println("startFILHO3");
                    System.out.print("------");
                    printSubTree(t.getChild().getSibling().getSibling());
                    //System.out.println("endFILHO3");
                }
            } catch (Exception e) {
                System.out.println(t.toString());
            }
        }
    }










}
