/**
 * This the abstract class used as a container 
 * for all the animation patterns that will 
 * exist.
 */

package Alma.animation.patterns;

import Alma.animation.*;
import Alma.TAB;

/**
 *
 * @author Nuno Oliveira
 */
public abstract class AnimationPattern {
    
    //This stores the object that will be animated by the pattern
    private Actor obj;
    //This Ids Table must be set everytime before the evaluation of the pattern
    private TAB idstab;
    private int[] poses;
    private boolean isAggregator;
    
    
    /**
     * This method is used to perform the animation
     * of the pattern. It is very similar to the run
     * method in the Runnable Class.
     */
    public abstract void animate();

    
    public AnimationPattern(Actor obj, int[] poses) {
        this.obj = obj;
        this.poses = poses;
    }

    public Actor getObj() {
        return obj;
    }

    public TAB getTab() {
        return idstab;
    }

    public void setObj(Actor obj) {
        this.obj = obj;
    }

    public void setTab(TAB t){
        this.idstab = t;
    }

    public int[] getPoses() {
        return poses;
    }

    public void setPoses(int[] poses) {
        this.poses = poses;
    }

    public boolean isIsAggregator() {
        return isAggregator;
    }

    public void setIsAggregator(boolean isAggregator) {
        this.isAggregator = isAggregator;
    }

    



    /**
     * This method gets the value of a variable from the identifier table.
     *
     * @param var The name of a variable to get the value from the id table
     * @return
     */
    public Object getValue(String var){
        Object value = null;
        //System.out.println(var);
        value = this.idstab.getTable(var);
        return value;
    }
    
    
    
}
