import Lisa.Parser.LR.LR1.*;
/*
 * Seminarska naloga pri predmetu
 *           Objektno in funkcijsko programiranje
 * 
 *      skelet za LR(1) parser oziroma template
 * 
 * Avtor : David Balazic
 */
import java.io.*;
import Lisa.Parser.*;
import Lisa.Interface.CMessage;
import Lisa.Scanner.*;
import java.util.Stack;
import java.util.zip.*;

/** CLR1Parser je glavni del LR(1) parserja. Vsebuje gradnjo stanj koncnega avtomata,
 * gradnjo krmilnih tabel ( matriki F in G ) , ter sam parser.
 * @version 0.7 <BR>
 * last change : 6-mar-98
 * @author David Balazic
 */

//public class CLR1Parser implements Parser
public class 
CParser
implements Parser
{
   // tip tokena  spada v CTokenInfo , ampak ne gre tja 
   /** spada v CTokenInfo , oznacuje da je tip tokena kategorija */
   
   public static final int cCategory = 3;  // vrednost izbrana "od oka"
   /** spada v CTokenInfo , oznacuje da je tip tokena terminal */
   public static final int cTerminal = 7;  
   
   /** interne informacije o tokenu */
   private class CTokenInfo
     {
	/** referenca na CToken */
	public CToken token;        // sam token
	/** oznaka napake */
	public boolean error;       // ce je prislo do napake
	/** LocalId ustreznega simbola */
	public int symbolId;
     }
   
   /** maksimalno stevilo stanj , ki jih vrinemo med obdelavo napake, ali stevilo
    nepopolnih produkcij, ki jih opravimo */
   int cMAX_ERRORS = 4 ;
   
   /* kode za operacije parserja */
   private final static int cOP_ERROR=-1;
   private final static int cOP_NONE=0;
   private final static int cOP_SHIFT=1;
   private final static int cOP_REDUCE=2;
   private final static int cOP_ACCEPT=3;
   
   /** krmilna tabela */
   private CLR1Table        controlTable;
   /** Podatki za reduciranje pri error recovery, 
    vsebuje stevilko najobetavnejse produkcije za vsako stanje */
   
   

   //  nekaj tabel ...
   static int errorReduceProduction[] = null;
   static int errorReduceAmount[] = null;
   static int errorReduceExpSymbol[] = null;
   static int trows = 31 , tcols = 28 ;
   static class NS { static int nextStateTableRLE[][] = null; }   static class OP { static int operationTableRLE[][] = null; }     static class PR { static int productionTableRLE[][]  = null; }   boolean isTerminalArray[] = {true,true,true,true,true,true,true,true,true,true,false,false,false,false,false,false,false,false,false,false,false,false,true,true,true,true,true,true};
   boolean termOrCatArray[] = {false,false,false,false,false,true,true,true,true,true,false,false,false,false,false,false,false,false,false,false,false,false,true,true,true,true,true,true};
   int symbolTypeArray[] = {3,3,3,3,3,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,1,1,1,1,1,1};
   boolean symbolEpsilonArray[] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true};
   boolean symbolNullableArray[] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,true,false,false,false,false,false,false,false,false,true};

   int prodLeftSideArray[] = {14,11,11,10,16,15,18,18,20,13,17,17,17,19,19,12,12,21};
   int prodSizeArray[] = {2,1,2,5,2,2,1,2,5,2,1,1,1,1,1,1,1,1};
   boolean prodIsEpsilonArray[] = {false,false,false,false,false,false,true,false,false,false,false,false,false,false,false,false,false,false};
   int nrProductions = 18 ;
     
   /** izvlecek podatkov o simbolih gramatike */
   private CLR1SymbolData symbols;
   /** izvlecek podatkov o produkcijah */
   private CLR1ProductionData productions;
   
   /** localID simbola $ */
   private int eofSymbolId;
   /** localID pomoznega startnega simbola */
   private int startSymbolId;
   /** localID simbola epsilon */
   private int epsilonID;
/** stevilo redukcij opravljenih pri odpravljanju napak */
   private int errorReductionCount = 0;
   /** stevilo vstavljanj opravljenih pri odpravljanju napak */
   private int errorInsertionCount = 0;
   /** predstavlja razmerje med redukcijami in vstavljanjem v primeru napake
    */
 private float errorReductionFactor =  2; // 2:1 za vstavljanje

       static int nextStateTable[][] = null;
       static int operationTable[][] = null;
       static int productionTable[][] = null;

//   public CLR1Parser()
  //   { // konstruktor
	
   public CParser()
     {
        int nrSymbols = 28 ;
        symbols = new CLR1SymbolData(nrSymbols);
        symbols.symbols[0].symbolName = "#Lavanda$Body";
        symbols.symbols[1].symbolName = "#Lavanda$Date";
        symbols.symbols[2].symbolName = "#Lavanda$House";
        symbols.symbols[3].symbolName = "#Lavanda$Identifier";
        symbols.symbols[4].symbolName = "#Lavanda$Number";
        symbols.symbols[5].symbolName = "$__JZTJETZJHTE32ZJEZWEEER43252543GSRETEHRJZXYXSD";
        symbols.symbols[6].symbolName = "(";
        symbols.symbols[7].symbolName = ")";
        symbols.symbols[8].symbolName = ",";
        symbols.symbols[9].symbolName = "-";
        symbols.symbols[10].symbolName = "Lavanda$BAG";
        symbols.symbols[11].symbolName = "Lavanda$BAGS";
        symbols.symbols[12].symbolName = "Lavanda$CLASS";
        symbols.symbols[13].symbolName = "Lavanda$HEADER";
        symbols.symbols[14].symbolName = "Lavanda$LAVANDA";
        symbols.symbols[15].symbolName = "Lavanda$LOT";
        symbols.symbols[16].symbolName = "Lavanda$LOTS";
        symbols.symbols[17].symbolName = "Lavanda$MATERIAL";
        symbols.symbols[18].symbolName = "Lavanda$OTHERS";
        symbols.symbols[19].symbolName = "Lavanda$TINT";
        symbols.symbols[20].symbolName = "Lavanda$TYPE";
        symbols.symbols[21].symbolName = "POMOZNI_ZACETNI_SIMBOL__JZTJETZJHTEZJEZWEEERUTZGHSDFGSRETEHRJZXYXSD";
        symbols.symbols[22].symbolName = "color";
        symbols.symbols[23].symbolName = "cotton";
        symbols.symbols[24].symbolName = "fiber";
        symbols.symbols[25].symbolName = "white";
        symbols.symbols[26].symbolName = "wool";
        symbols.symbols[27].symbolName = "epsilon";

        eofSymbolId = 5;
        startSymbolId = 21;
        epsilonID = 27;

        controlTable = new CLR1Table(trows,tcols);
	int r,c;
	for (r=0;r<trows;r++)
	    for(c=0;c<tcols;c++)
	  {
	     controlTable.tabela[r][c].nextState = new Integer(nextStateTable[r][c]);
	     controlTable.tabela[r][c].operation = operationTable[r][c];
	     controlTable.tabela[r][c].productionNumber = productionTable[r][c];
	  }
	
	int i;
	for (i=0;i<nrSymbols;i++)
	  {
	     symbols.symbols[i].isTerminal = isTerminalArray[i];
	     symbols.symbols[i].termOrCategory = termOrCatArray[i];
	     if ( symbolTypeArray[i] == 1 )
	       symbols.symbols[i].parseSymbol = new CParseSymbolTerm(symbols.symbols[i].symbolName,
							 i,symbolEpsilonArray[i],symbolNullableArray[i]);
	     else if ( symbolTypeArray[i] == 2 )
	       symbols.symbols[i].parseSymbol = new CParseSymbolNonTerm(symbols.symbols[i].symbolName,
									i,symbolNullableArray[i]);
	     else if ( symbolTypeArray[i] == 3 )
	       symbols.symbols[i].parseSymbol = new CParseSymbolCategory(symbols.symbols[i].symbolName,
									i,symbolNullableArray[i]);
	  }

	productions = new CLR1ProductionData(nrProductions);
	for  ( i=0 ; i<nrProductions ; i++)
	  {
	     productions.productions[i].leftSideID = prodLeftSideArray[i];
	     productions.productions[i].size = prodSizeArray[i];
	     productions.productions[i].isEpsilon = prodIsEpsilonArray[i];
	  }
     }
   
   /** Ali je gramatika LR(1) in ni prislo do drugih napak med inicializacijo ?<BR>
    @return <UL>
    <LI> true - parser je pripravljen za parsanje
    <LI> false - parser ni pripravljen
    </UL>
    */
   public boolean canParse()
     // ali smo zmozni parsati dano gramatiko ?
     {
	return true; 
     }
   
   /** Vrni ime parserja
    @return vrne se niz   "LR(1)" 
    */
   public String getParserName()
     /* vrni ime/tip parserja */
     {
	return "LR(1)";
     }
   
 /** parsaj vhodni tok
       @param aLexicalAnalysis lexikalno analizator za vhod
       @return sintakticno drevo ali null v primeru napake */
   public CSyntaxTree parse(LexicalAnalysis aLexicalAnalysis)
     /* razpoznaj vhodni tok tokenov in vrni sintakticno drevo */
     {
	Stack stateStack = new Stack();  // sklad stanj avtomata
	Stack treeStack = new Stack();   // sklad simbolov ( CTreeNode )
	Stack helpStack = new Stack();   // pomozni sklad ( vstavljanje CTreeNode-ov v starsevski
	//                 CTreeNode v obratnem vrstnem redu ,
	//		    kot jih dobimo iz treeStack-a ) 
	int curState;            // trenutno stanje ( vrh sklada )
	int curToken;            // trenutni token na vhodu
	int operation;           // operacija shift,reduce,accept,error
	int u;                   // stevec v zanki
	int m;                   // meja iteracij v zanki
	int prodNum;             // stevilka produkcije pri redukciji
	int leftSideID;          // LocalID leve strani produkcije
	
	boolean errormode = false ; // ali vhod ne ustreza gramatiki ?  potem se ne ukvarjamo z sint. drevesom
	int errorcount=0;        // stevilo umetno vstavljenih stanj 
	int wantedTokenId;       // token , ki ga pricakujemo - obdelava napak 
	Integer nextState;       // umetno vstavljeno stanje
	
	CTreeNode newTreeNode;   // vozlisca , ki jih bomo kreirali
	
	CTokenInfo tokenInfo = new CTokenInfo();  // informacije o tokenu
	
	CLR1Tabentry tabela[][];            // bliznjica do kontrolne tabele parserja
	tabela = controlTable.tabela;
	
	setUpCategoryIDs(aLexicalAnalysis); 
	
	CToken token =  aLexicalAnalysis.nextToken();  // poberi prvi token
	curState = 0;                                  // zacetno stanje 
	tokenToId(tokenInfo,token);    // pretvori podatke o tokenu v prirocno obliko
	
	stateStack.push(new Integer(0));               // porini na sklad prvo stanje
	
	while(true)          // "neskoncna" zanka
	  {
	     if ( tokenInfo.error )    // ce je scanner vrnil napako
	       {
		  wantedTokenId = selectWantedTokenId(curState);  // pricakovan token ...
		  if ( errorcount == 0 )  // smo na zacetku obdelave napake
		    {
		       if ( wantedTokenId != -1 )   // ne vemo kaj smo pricakovali
			 CMessage.getMessage().printError("neznan token , pricakoval : '"+
					     symbols.getParseSymbol(wantedTokenId).getName()+"'",
					     token.row(),token.column(),token.fileName());
		       else
			 CMessage.getMessage().printError("neznan token",
					     token.row(),token.column(),token.fileName());
		    }
		  
		  if ( ( token.type() == -1 ) && ( !token.isEOF() ) )
		    return null;  // ce se je scannerju zataknilo, koncamo

		  errormode=true; // vhod ni pravilen
		  token =  aLexicalAnalysis.nextToken();  // poberi naslednji token
		  tokenToId(tokenInfo,token); // pretvori podatke o tokenu v prirocno obliko
		  continue;            // in poskusi ponovno
	       }
	     curToken = tokenInfo.symbolId;      // koda tokena
	     
	     // ugotovi operacijo
	     switch ( tabela[curState][curToken].operation )
	       {
		case cOP_SHIFT :  // shift
		  errorcount=0;  // situacija spet normalna
		  stateStack.push(controlTable.destination(curState,curToken));
		  // naslednje stanje na sklad
		  curState = controlTable.destination(curState,curToken).intValue();
		  // novo stanje postane trenutno stanje
		  if ( !errormode )
		    {  // v primeru napake se ne ukvarjaj z drevesom, sicer pa
		       newTreeNode =  new CTreeNode(symbols.getParseSymbol(tokenInfo.symbolId));
		       newTreeNode.setToken(token);  // kreiramo novi TreeNode
		       treeStack.push(newTreeNode);  // in ga posljemo na sklad
		    }
		  token =  aLexicalAnalysis.nextToken();
		  tokenToId(tokenInfo,token);
		  break;
		case cOP_REDUCE : // reduce
		  errorcount=0;  // situacija spet normalna
		  prodNum = controlTable.getProductionNumber(curState,curToken);
		  leftSideID = productions.getLeftSide(prodNum);
		  
		  if (productions.isEpsilon(prodNum))
		    { // kreiraj simbol epsilon
		       CTreeNode epsTreeNode =  new CTreeNode(symbols.getParseSymbol(epsilonID));
		       epsTreeNode.setToken(null);
		       newTreeNode = new CTreeNode(symbols.getParseSymbol(leftSideID));
		       newTreeNode.addNode(epsTreeNode);
		       newTreeNode.setProdNumber(prodNum);
		    }
		  else
		    {
		       newTreeNode = new CTreeNode(symbols.getParseSymbol(leftSideID));
		       newTreeNode.setProdNumber(prodNum);
		       m = productions.getSize(prodNum);

		       if ( ! errormode ) // v primeru napake se ne ukvarjaj z drevesom
			 {
			    for (u=0;u<m;u++)
			      {
				 stateStack.pop();
				 helpStack.push(treeStack.pop());
			      }
			    for (u=0;u<m;u++)
			      newTreeNode.addNode((CTreeNode)helpStack.pop());
			 }
		       else // zgodila se je napaka , pozabi na sint. drevo
			 for (u=0;u<m;u++)
			   stateStack.pop();
		    }
		  curState=((Integer)stateStack.peek()).intValue();
		  stateStack.push( controlTable.destination(curState,leftSideID) );
		  curState = controlTable.destination(curState,leftSideID).intValue() ;
		  if ( ! errormode ) // v primeru napake se ne ukvarjaj z drevesom
		    treeStack.push(newTreeNode);
		  break;
		case cOP_ACCEPT : // accept
		  
		  if ( errormode ) // v primeru napake se ne ukvarjaj z drevesom
		    {
		       return null;
		    }
		  CSyntaxTree sTree=new CSyntaxTree();
		  sTree.setRoot (((CTreeNode)treeStack.peek()));
		  return sTree;
		case cOP_ERROR :// error
		  if (errorcount==0) // ce je do zdaj bilo OK , potem izpisi sporocilo
		    {
		       wantedTokenId = selectWantedTokenId(curState);
		       if ( wantedTokenId != -1 )
			 CMessage.getMessage().printError("napaka pri razpoznavanju, pricakoval token : '"+
					     symbols.getParseSymbol(wantedTokenId).getName()+"'",
					     token.row(),token.column(),token.fileName());
		       else
			 CMessage.getMessage().printError("napaka pri razpoznavanju",token.row(),token.column(),token.fileName());
		       errormode=true;
		    }
		  errorcount++;
		  
		  if ( errorcount <= cMAX_ERRORS)
		    {
		       if ( errorReduceProduction[curState]!=-1 && ((errorReductionCount*errorReductionFactor < errorInsertionCount) || selectNextStateOnError(curState)==null) )
			 {
			    errorReductionCount++;
			    wantedTokenId = errorReduceExpSymbol[curState];
			    prodNum = errorReduceProduction[curState];
			    leftSideID = productions.getLeftSide(prodNum);
			    if (leftSideID == startSymbolId )
			      // reduciramo na startni simbol , torej nimamo vec kaj poceti
			      return null;
			    
			    m = errorReduceAmount[curState];
			    for (u=0;u<m;u++)
			      stateStack.pop();
			    
			    curState=((Integer)stateStack.peek()).intValue();
			    stateStack.push( controlTable.destination(curState,leftSideID) );
			    curState = controlTable.destination(curState,leftSideID).intValue() ;
			    if (wantedTokenId!=-1)
			      CMessage.getMessage().printError("mozen token : '"+
						  symbols.getParseSymbol(wantedTokenId).getName()+"'",
						  token.row(),token.column(),token.fileName());
			    
			    break;			    
			 }

		       nextState=selectNextStateOnError(curState);
		       if ( nextState != null )
			 {
			    errorInsertionCount++;
			    CMessage.getMessage().printError("mozen token : '"+
				  symbols.getParseSymbol(selectWantedTokenId(curState)).getName()+"'",
				  token.row(),token.column(),token.fileName());
			    
			    stateStack.push(nextState);
			    curState=nextState.intValue();
			    break;
			 }
		    }
		  // tu ni else-ta !
		    { // konec "igre" , iscemo dolocen token , preskakujemo vhod
		       wantedTokenId = selectWantedTokenId(curState);
		       // kaj ce wanted == -1 !!!!! ???? --> pridemo do konca vhoda ...
		       while ( tokenInfo.symbolId != wantedTokenId )
			 {
			    token =  aLexicalAnalysis.nextToken();
			    tokenToId(tokenInfo,token);
			    if ( tokenInfo.symbolId == eofSymbolId )
			      return null; // prisli na konec vhoda
			 }
		       break;
		    }
		default: // sem ne smemo priti
		  CMessage.getMessage().error("interna napaka 1 , stanje : "+curState+" , token : "+token.name());
		  return null;
	       }
	  }
     }

   /** pretvori token v prirocno obliko informacije 
    @param aInfo tu se vrnejo rezultati
    @param aToken vhodni podatki
   */
   private void tokenToId(CTokenInfo aInfo,CToken aToken)
     {// pretvori token v uporabne informacije    XXXXX za optimizirat !!!!
	aInfo.token = aToken;            // postavi kazalec na CToken
	aInfo.error = false;             // do sedaj se ni napake
	if ( aToken.isEOF() && aToken.type()==-1 )
	  {  // ce konec vhoda
	     aInfo.symbolId = eofSymbolId ;
	     return;
	  }
	
	int i,        // stevec v zanki
	  type,       // tip tokena
	  symbolcount=symbols.size();  // stevilo vseh znanih simbolov
	int categoryID=-1;
	
	for (i=0 ; i<symbolcount; i++ )
	  { // primerjamo z imeni simbolov
	     if ( !symbols.getIsTerminal(i) ) //  ce neterninal -> naslednji simbol
	       continue;
	     if ( symbols.getParseSymbol(i).compareToToken(aToken) )
	       {
		  if (symbols.getTermOrCategory(i) )
		    { // navaden terminal -> return
		       aInfo.symbolId = i; // bnf.elementAt(i).getLocalId()   ==   i    !!!
		       return;
		    }
		  else
		    { // kategorija , pogledamo se ce ustreza kaksnemu terminalu
		       // preveri ali je ta vhodni simbol smiseln
		       categoryID = i;
		    }
	       }
	  }
	aInfo.symbolId = categoryID;
	if ( categoryID != -1 )
	  return; // nasli kategorijo -> vrni OK
	// sicer napaka
	aInfo.error = true; 
	return;
     }
/**  postavi categoryID za CParseSymbol-e
 @param aLexicalAnalysis lexikalni analizator */  
   void setUpCategoryIDs(LexicalAnalysis aLexicalAnalysis)
     { // postavi categoryID za CParseSymbol-e
	for ( int i = 0 ; i < symbols.symbols.length ; i++ )
	  {
	     if ( symbols.getIsTerminal(i) && !symbols.getTermOrCategory(i) )
	       { // dobili smo kategorijo ...
		  ((CParseSymbolCategory)symbols.getParseSymbol(i)).setCategoryId
		    ( aLexicalAnalysis.getCategoryId(symbols.getName(i).substring(1,
						       symbols.getName(i).length()  )));
	       }
	  }
     }
   
   /** izbere naslednje stanje v primeru napake.
    Izbere se stanje v katerega gre najvec prehodov iz trenutnega. 
    @param aCurState trenutno stanje parserja
    @return stevilka naslednjega stanja
    */
   Integer selectNextStateOnError(int aCurState)
     {
	int nextStates[] = new int [controlTable.rows];  // int za vsa stanja
	int i;  // stevec zanke
	
	for ( i=0 ; i<controlTable.rows ; i++)
	  nextStates[i]=0;   // brisi polje
	
	int symbolMax = symbols.size();  // stevilo simbolov
	
	for (i=0 ; i<symbolMax ; i++ )  // skozi vse simbole
	  if (controlTable.operation(aCurState,i)== cOP_SHIFT)  // ce obstaja prehod pri tem simbolu
	    nextStates[controlTable.destination(aCurState,i).intValue()]++;  // belezimo prehod v neko stanje
	
	int nextState=-1;
	int max = 0;
	
	// poiscemo stanje v katerega gre najvec prehodov
	for ( i=0 ; i<controlTable.rows ; i++ )
	  if ( nextStates[i] > max ) {
	     max = nextStates[i];
	     nextState = i;
	  }
	if ( nextState == -1 )
	  return null;   // nismo nasli nobenega stanja
	
	return new Integer(nextState);  // vrni stevilko stanja
     }
   
   
 /** ugotovi pricakovan token za trenutno stanje
  @param aCurState trenutno stanje parserja
  @return id pricakovanega tokena
  */
   int selectWantedTokenId(int aCurState)
     {
	int nextStates[] = new int [controlTable.rows]; // stevilo prehodov v doloceno stanje
	int i;    // stevec v zanki
	
	// inicializiraj polje , postavi elemente na 0
	for ( i=0 ; i<controlTable.rows ; i++)
	  nextStates[i]=0;
	
	int symbolMax = symbols.size(); // stevilo simbolov
	
	// prestej prehode v vsako stanje
	for (i=0 ; i<symbolMax ; i++ )
	  {
	     if (controlTable.operation(aCurState,i) == cOP_SHIFT)
	       nextStates[controlTable.destination(aCurState,i).intValue()]++;
	  }
	
	// poisci stanje v katerega se gre najvec krat
	int nextState=-1;  //  stanje v katerega gre najvec prehodov
	int max = 0;  // trenutni maximum stevila prehodov
	for ( i=0 ; i<controlTable.rows ; i++ )
	  if ( nextStates[i] > max ) {
	     max = nextStates[i];
	     nextState = i;
	  }
	// ni nobenega nasledenjga stanja , vrni -1
	if ( nextState == -1 )
	  return -1;
	
	// poisci in vrni prvi simbol , ki povzroci prehod v naj... stanje
	for (i=0 ; i<symbolMax ; i++ )
	  if (controlTable.operation(aCurState,i) == cOP_SHIFT)
	    if ( controlTable.destination(aCurState,i).intValue() == nextState)
	      return i;
	return -1;
     }
     
     private static void decompress(int[][] result, int[][] RLE)
      {
        for (int i = 0; i<RLE.length; i++)
	 {
	   int a = 0;
	   for (int j = 0; j<RLE[i].length; j+=2)
	    {
	      for (int k =0; k<RLE[i][j]; k++)
	        result[i][a++] = RLE[i][j+1];
	    }
	 }
      } 

static class EC0 { static byte[] encodedString={31,-117,8,0,0,0,0,0,0,0,-19,-102,57,82,-61,48,20,-122,101,-123,-80,-123,44,-20,59,-23,56,5,23,72,-63,112,-128,52,28,1,-104,-127,-95,-94,-89,-30,20,116,84,-108,-100,-121,-118,25,106,-54,-96,7,-49,65,17,66,-111,-91,-9,108,101,-16,63,-13,-58,-78,44,-23,-45,98,109,-74,-98,-34,69,-13,-6,74,-56,-31,-32,-12,-27,-4,-8,-26,-19,-7,81,10,113,123,33,-124,-24,43,91,85,-74,36,-66,-107,-69,27,-54,50,-12,3,-73,-60,107,87,89,79,-39,-94,-78,57,-12,107,-30,-3,-126,-59,-81,-93,-84,-115,-49,22,-47,-106,-107,-83,40,107,93,95,-118,-69,47,66,31,73,-103,102,-46,112,-25,-42,48,-62,101,72,-43,-29,72,-52,-127,25,94,98,-40,-90,30,127,34,23,29,-116,-47,26,-95,-32,94,115,-49,-29,-75,-89,-7,65,125,-83,105,-27,95,-41,-97,-25,97,-116,-5,-43,-111,33,-43,60,-115,-31,112,-80,-3,-15,122,-74,-5,112,127,50,110,-97,113,-18,14,48,102,102,-58,116,41,-81,-125,34,113,-58,57,-102,32,107,41,-50,-123,-92,22,79,70,122,-45,22,3,90,6,90,-128,-113,-52,-83,-44,-56,-48,87,-85,33,-105,-95,-96,55,108,-55,-26,79,71,-42,72,-53,20,-92,-30,100,46,-3,-112,-13,-111,21,102,0,-97,-104,48,123,104,110,24,-99,-69,-58,-13,-34,-17,88,54,114,72,-66,99,52,43,100,-37,92,-60,79,-122,25,-45,39,69,-104,77,-23,-56,-22,-7,-122,79,106,69,-28,28,73,54,-87,105,126,-28,24,-87,-12,-74,52,-9,-74,-26,-34,-7,77,38,-24,-49,-69,-58,-13,-70,63,-49,40,89,-123,-37,-85,-122,76,-87,-102,28,43,-107,-50,-2,31,-2,48,94,28,-94,-5,-120,-98,92,68,53,-71,116,50,-20,-115,-29,-9,-77,89,-47,56,19,121,-80,-19,115,-126,82,-117,39,-69,-24,49,-27,-12,35,35,37,-24,-53,64,28,89,-7,73,46,-22,-33,-28,-54,-37,89,114,-106,-37,-3,-122,-39,-56,84,-11,-31,-75,-97,101,-87,-7,20,106,59,47,-101,111,25,-11,112,-74,62,62,45,-99,-62,43,50,-78,58,-104,33,50,-39,-37,-26,-67,-85,-108,33,-19,79,66,38,-17,89,-59,-25,42,126,114,-116,-52,30,103,115,-89,-34,-97,57,-58,-78,20,70,79,87,40,-22,92,64,90,-87,-113,36,26,-7,-33,-116,-37,-27,-66,-37,33,-14,89,-93,-25,-41,-44,86,-67,53,-103,-111,108,-39,-49,-106,-87,-102,-100,6,89,16,-3,25,44,78,-26,-42,-44,93,37,-37,122,112,-22,119,11,81,13,-103,83,-87,-81,-56,-86,39,11,-29,-17,111,-87,100,-25,127,41,62,50,-91,-68,87,100,-13,-43,-112,57,84,-109,-45,35,11,-122,51,100,-50,-47,19,78,-12,85,-16,-91,-37,-56,5,-100,58,-76,-98,45,10,17,-92,-107,122,59,-25,82,-31,58,-107,-111,-37,-45,67,81,-110,5,-63,41,-60,48,50,-121,-100,-67,-118,-11,124,-96,-109,-52,114,62,-50,-117,-20,117,-54,-123,-123,-36,-78,-59,-96,-46,39,107,90,-75,104,33,46,0,0};};

static class EC { static byte[][] encodedString = {EC0.encodedString}; static int length=647;};

     static {
       if (nextStateTable!=null) {
       nextStateTable = new int[trows][tcols];
       operationTable = new int[trows][tcols];
       productionTable = new int[trows][tcols];
       decompress(nextStateTable, NS.nextStateTableRLE);
       decompress(operationTable, OP.operationTableRLE);
       decompress(productionTable, PR.productionTableRLE);
       } else {
         try {

           InputStream res = null;
           if (EC.encodedString==null)
		      res = CTokenInfo.class.getResourceAsStream("LR1Tables.dat");
		    else {
		      byte[] data = new byte[EC.length];
		      int pos = 0;
		      for (int i = 0; i<EC.encodedString.length; i++) {
		      	System.arraycopy(EC.encodedString[i],0,data,pos,EC.encodedString[i].length);
		      	pos +=EC.encodedString[i].length;
		      }
		      res = new ByteArrayInputStream(data);
		    }
			ObjectInputStream ois = new ObjectInputStream(new GZIPInputStream(res));
			errorReduceProduction = (int[])ois.readObject();
			errorReduceAmount = (int[])ois.readObject();
            errorReduceExpSymbol = (int[])ois.readObject();
			nextStateTable = (int[][])ois.readObject();           
			operationTable = (int[][])ois.readObject();
			productionTable = (int[][])ois.readObject();
         } catch (Exception e) {
           e.printStackTrace();
         }
       }
     }
}
// to je zadnja vrstica template-ja
